#include "Global.h"
#include "VDXMLAux.h"
#include "VDLogFactory.h"

Global* global = NULL;

Global::Global()
{
  load();
  initializeLog();
}

Global::~Global()
{
  save();
  finalizeLog();
}

void Global::load()
{
  xmlFile.loadFromFile();
}

void Global::save()
{
  xmlFile.saveToFile();
}

bool Global::isOk()
{
  bool res = true;

  SnoopARPSpoof arpSpoof;
  loadARPSpoof(&arpSpoof);
  if (arpSpoof.adapterIndex == SNOOP_INVALID_ADAPTER_INDEX) res = false;

  SnoopNetInfo netInfo;
  loadNetInfo(&netInfo);
  if (netInfo.ip == 0) res = false;
  if (netInfo.mac.isClear()) res = false;
  if (netInfo.subnetMask == 0) res = false;
  if (netInfo.gateway == 0) res = false;
  if (netInfo.startIP == 0) res = false;
  if (netInfo.endIP == 0) res = false;

  return res;
}

void Global::initializeLog()             
{
  VDXML* xml = xmlFile.root->gotoChild("Log");
  VDLog* log = VDLogFactory::createByXML(xml);

  if (log == NULL)
  {
    VDLogList* logList = new VDLogList;
    logList->items.add(VDLogFactory::createByURI("udp"));
    logList->items.add(VDLogFactory::createByURI("file://log/"));
    log = logList;
    VDLogFactory::save(log, xml);
    save();
  }
  VDLog::changeLog(log);
}

void Global::finalizeLog()
{
  VDLog::changeLog(NULL);
}

bool Global::loadWindowPos(HWND hwnd, char* name)
{
  VDXML* xml = xmlFile.root->gotoChild("WindowPos")->gotoChild(name);
  return VDXMLAux::loadWindowPos(hwnd, xml);
}

bool Global::saveWindowPos(HWND hwnd, char* name)
{
  VDXML* xml = xmlFile.root->gotoChild("WindowPos")->gotoChild(name);
  return VDXMLAux::saveWindowPos(hwnd, xml);
}

void Global::loadARPSpoof(SnoopARPSpoof* arpSpoof)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopARPSpoof");
  arpSpoof->load(xml);
}

void Global::saveARPSpoof(SnoopARPSpoof* arpSpoof)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopARPSpoof");
  arpSpoof->save(xml);
}

void Global::loadNetInfo(SnoopNetInfo* netInfo)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopNetInfo");
  netInfo->load(xml);
}

void Global::saveNetInfo(SnoopNetInfo* netInfo)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopNetInfo");
  netInfo->save(xml);
}

void Global::loadSessionListFrame(TSnoopSessionListFrame* frame)
{
  VDXML* xml = xmlFile.root->gotoChild("SessionListFrame");
  frame->load(xml);
}

void Global::saveSessionListFrame(TSnoopSessionListFrame* frame)
{
  VDXML* xml = xmlFile.root->gotoChild("SessionListFrame");
  frame->save(xml);
}
