#include "Global.h"
#include "VDXMLAux.h"
#include "VDLogFactory.h"

Global* global = NULL;

Global::Global()
{
  load();
  initializeLog();
}

Global::~Global()
{
  save();
  finalizeLog();
}

void Global::load()
{
  xmlFile.loadFromFile();
}

void Global::save()
{
  xmlFile.saveToFile();
}

bool Global::isOk()
{
  bool res = true;

  loadAdapter(&adapter);
  if (adapterIndex == -1) res = false;

  loadNetInfo(&netInfo);
  if (netInfo.ip == 0) res = false;
  if (netInfo.mac.isClear()) res = false;
  if (netInfo.subnetMask == 0) res = false;
  if (netInfo.gateway == 0) res = false;
  if (netInfo.startIP == 0) res = false;
  if (netInfo.endIP == 0) res = false;

  return res;
}

void Global::initializeLog()             
{
  VDXML* xml = xmlFile.root->gotoChild("Log");
  VDLog* log = VDLogFactory::createByXML(xml);

  if (log == NULL)
  {
    VDLogList* logList = new VDLogList;
    logList->items.add(VDLogFactory::createByURI("udp"));
    logList->items.add(VDLogFactory::createByURI("file://log/"));
    log = logList;
    VDLogFactory::save(log, xml);
    save();
  }
  VDLog::changeLog(log);
}

void Global::finalizeLog()
{
  VDLog::changeLog(NULL);
}

bool Global::loadWindowPos(HWND hwnd, char* name)
{
  VDXML* xml = xmlFile.root->gotoChild("WindowPos")->gotoChild(name);
  return VDXMLAux::loadWindowPos(hwnd, xml);
}

bool Global::saveWindowPos(HWND hwnd, char* name)
{
  VDXML* xml = xmlFile.root->gotoChild("WindowPos")->gotoChild(name);
  return VDXMLAux::saveWindowPos(hwnd, xml);
}

void Global::loadAdapter(SnoopAdapter* adapter)
{
  VDXML* xml = xmlFile.root->gotoChild("Adapter");
  adapter->load(xml);
}

void Global::saveAdapter(SnoopAdapter* adapter)
{
  VDXML* xml = xmlFile.root->gotoChild("Adapter");
  adapter->save(xml);
}

int Global::getAdapterIndex()
{
  return adapter.adapterIndex;
}

void Global::setAdapterIndex(int value)
{
  adapter.adapterIndex = value;
  netInfo.adapterIndex = value;
}

void Global::loadNetInfo(SnoopNetInfo* netInfo)
{
  VDXML* xml = xmlFile.root->gotoChild("NetInfo");
  netInfo->load(xml);
}

void Global::saveNetInfo(SnoopNetInfo* netInfo)
{
  VDXML* xml = xmlFile.root->gotoChild("NetInfo");
  netInfo->save(xml);
}

void Global::loadHostListFrame(TSnoopHostListFrame* frame)
{
  VDXML* xml = xmlFile.root->gotoChild("HostListFrame");
  frame->load(xml);
}

void Global::saveHostListFrame(TSnoopHostListFrame* frame)
{
  VDXML* xml = xmlFile.root->gotoChild("HostListFrame");
  frame->save(xml);
}


