#include "Global.h"
#include "VDXMLAux.h"
#include "VDLogFactory.h"

Global* global = NULL;

Global::Global()
{
  load();
  initializeLog();
}

Global::~Global()
{
  save();
  finalizeLog();
}

void Global::load()
{
  xmlFile.loadFromFile();
}

void Global::save()
{
  xmlFile.saveToFile();
}

void Global::initializeLog()             
{
  VDXML* xml = xmlFile.root->gotoChild("Log");
  VDLog* log = VDLogFactory::createByXML(xml);

  if (log == NULL)
  {
    VDLogList* logList = new VDLogList;
    logList->items.add(VDLogFactory::createByURI("udp"));
    logList->items.add(VDLogFactory::createByURI("file://log/"));
    log = logList;
    VDLogFactory::save(log, xml);
    save();
  }
  VDLog::changeLog(log);
}

void Global::finalizeLog()
{
#ifndef VD_USE_AUTO_CLASS_INITIALIZER
  VDLog::changeLog(NULL);
#endif  VD_USE_AUTO_CLASS_INITIALIZER
}

void Global::loadTCPServer(VDTCPServer* tcpServer)
{
  VDXML* xml = xmlFile.root->gotoChild("VDTCPServer");
  tcpServer->load(xml);
}

void Global::saveTCPServer(VDTCPServer* tcpServer)
{
  VDXML* xml = xmlFile.root->gotoChild("VDTCPServer");
  tcpServer->save(xml);
}

void Global::loadARPRecover(SnoopARPRecover* arpRecover)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopARPRecover");
  arpRecover->load(xml);
}

void Global::saveARPRecover(SnoopARPRecover* arpRecover)
{
  VDXML* xml = xmlFile.root->gotoChild("SnoopARPRecover");
  arpRecover->save(xml);
}
