#include <stdio.h>
#include <conio.h>

#include "VDXMLFile.h"
#include "VDFactory.h"
#include "SnoopCapture.h"
#include "SnoopCaptureFactory.h"

void capture_OnCapture(SnoopCapture* capture, SnoopPacket* packet)
{
  printf("%d bytes captured\n", packet->pktHdr->caplen);
}

void main()
{
  VDXMLFile xmlFile;
  SnoopCapture* capture = NULL;

  //
  // Load capture
  //
  if (xmlFile.loadFromFile())
  {
    VDXML* xml = xmlFile.root->gotoChild("Capture");
    string className = xml->getStr("class", "");
    if (className != "")
    {
      capture = SnoopCaptureFactory::createByClassName(className);
      if (capture == NULL)
      {
        printf("Can not create capture for class name(%s)\n", className.c_str());
        goto _end;
      }
      capture->load(xml);
    }
  }

  //
  // Create and save capture
  //
  if (capture == NULL)
  {
    int i = 0;
    VDMetaClassList list;
    SnoopCaptureFactory::getMetaClassList(&list);
    for (VDMetaClassList::iterator p = list.begin(); p != list.end(); p++)
    {
      VDMetaClass* metaClass = *p;
      printf("%d : %s\n", i, metaClass->className());
      i++;
    }

    printf("Select capture : ");
    int select;
    scanf_s("%d", &select);

    i = 0;
    VDMetaClassList::iterator p = list.begin();
    while (i < select && p != list.end())
    {
      i++;
      p++;
    }
    if (p == list.end())
    {
      printf("Invalid index\n");
      goto _end;
    }

    VDMetaClass* metaClass = *p;
    capture = (SnoopCapture*)metaClass->createInstance();
    if (capture == NULL)
    {
      printf("Can not create capture for class name(%s)\n", metaClass->className());
      goto _end;
    }

    VDXML* xml = xmlFile.root->gotoChild("Capture");
    xml->setStr("class", capture->className());
    capture->save(xml);
    xmlFile.saveToFile();
    printf("You have to configure %s file to use properly\n", xmlFile.getDefaultFileName().c_str());
    goto _end;
  }

  //
  // Open
  //
  capture->onCapture = capture_OnCapture;
  if (!capture->open())
  {
    printf("%s\n", capture->error.msg);
    goto _end;
  }

  printf("press any key to close\n");
  _getch();
  capture->close();

_end:
  if (capture != NULL)
    delete capture;
}