#ifndef __SNOOPDLL_H__
#define __SNOOPDLL_H__

#ifdef  SNOOPDLL_EXPORTS
#define SNOOPDLL_API __declspec(dllexport)
#else
#define SNOOPDLL_API __declspec(dllimport)
#endif

#include <winsock2.h>
#include <windows.h>
#include <Iphlpapi.h>

#define  WPCAP
#define  HAVE_REMOTE
#include "pcap.h"

#include "SnoopCommon.h"

// ----------------------------------------------------------------------------
// Header
// ----------------------------------------------------------------------------
// packet
typedef struct pcap_pkthdr             PKT_HDR;

// data link layer
typedef struct libnet_ethernet_hdr     ETH_HDR;

// network layer
typedef struct libnet_ipv4_hdr         IPV4_HDR;
typedef struct libnet_ipv6_hdr         IPV6_HDR;
typedef struct libnet_arp_hdr          ARP_HDR;

// transport layer
typedef struct libnet_tcp_hdr          TCP_HDR;
typedef struct libnet_udp_hdr          UDP_HDR;
typedef struct libnet_icmpv4_hdr       ICMPV4_HDR;
typedef struct libnet_icmpv6_hdr       ICMPV6_HDR;

// ----------------------------------------------------------------------------
// Constant
// ----------------------------------------------------------------------------
#define SNOOP_DEFAULT_READTIMEOUT      1
#define SNOOP_DEFAULT_SNAPLEN          1600
#define SNOOP_DEFAULT_TIMEOUT          5000
#define SNOOP_INVALID_ADAPTER_INDEX    -1

// ----------------------------------------------------------------------------
// snoop_adapter_info_t
// ----------------------------------------------------------------------------
class snoop_adapter_info_t;
SNOOPDLL_API int                   snoop_get_adapter_count       ();
SNOOPDLL_API snoop_adapter_info_t* snoop_get_adapter_info        (int index);
SNOOPDLL_API char*                 snoop_get_adapter_name        (snoop_adapter_info_t* adapter_info);
SNOOPDLL_API char*                 snoop_get_adapter_description (snoop_adapter_info_t* adapter_info);
SNOOPDLL_API pcap_if_t*            snoop_get_adapter_dev         (snoop_adapter_info_t* adapter_info);

// ----------------------------------------------------------------------------
// snoop_t_on_capture
// ----------------------------------------------------------------------------
class snoop_t;
typedef void (*snoop_t_on_capture)(snoop_t* snoop, PKT_HDR* pkt_hdr, char* data);

// ----------------------------------------------------------------------------
// snoop_t
// ----------------------------------------------------------------------------
class snoop_t {};
SNOOPDLL_API void  snoop_get_owner        (snoop_t* snoop);
SNOOPDLL_API void  snoop_set_owner        (snoop_t* snoop, void* value);
SNOOPDLL_API bool  snoop_get_active       (snoop_t* snoop);
SNOOPDLL_API void  snoop_get_error        (snoop_t* snoop, char* buf, int len);
SNOOPDLL_API void  snoop_get_filter       (snoop_t* snoop);
SNOOPDLL_API void  snoop_set_filter       (snoop_t* snoop, char* buf, int len);
SNOOPDLL_API int   snoop_get_snap_len     (snoop_t* snoop);
SNOOPDLL_API void  snoop_set_snap_len     (snoop_t* snoop, int val);
SNOOPDLL_API int   snoop_get_flags        (snoop_t* snoop);
SNOOPDLL_API void  snoop_set_flags        (snoop_t* snoop, int val);
SNOOPDLL_API int   snoop_get_read_timeout (snoop_t* snoop);
SNOOPDLL_API void  snoop_set_read_timeout (snoop_t* snoop, int val);
SNOOPDLL_API void  snoop_get_source_name  (snoop_t* snoop, char* buf, int len);
SNOOPDLL_API bool  snoop_get_auto_capture (snoop_t* snoop);
SNOOPDLL_API void  snoop_set_auto_capture (snoop_t* snoop, bool value);
SNOOPDLL_API bool  snoop_get_thread_safe  (snoop_t* snoop);
SNOOPDLL_API void  snoop_set_thread_safe  (snoop_t* snoop, bool value);
SNOOPDLL_API pcap* snoop_get_pcap         (snoop_t* snoop);
SNOOPDLL_API int   snoop_get_data_link    (snoop_t* snoop);
SNOOPDLL_API void  snoop_set_on_capture   (snoop_t* snoop, snoop_t_on_capture* on_capture);
SNOOPDLL_API bool  snoop_open             (snoop_t* snoop);
SNOOPDLL_API bool  snoop_close            (snoop_t* snoop, bool wait = true);
SNOOPDLL_API int   snoop_read             (PKT_HDR** pktHdr, char** data);

// ----------------------------------------------------------------------------
// snoop_adapter_t
// ----------------------------------------------------------------------------
class snoop_adapter_t : public snoop_t {};
SNOOPDLL_API snoop_adapter_t* snoop_adapter_create();
SNOOPDLL_API void             snoop_adapter_destroy(snoop_adapter_t* adapter);
SNOOPDLL_API int              snoop_adapter_get_index(snoop_adapter_t* adapter);
SNOOPDLL_API void             snoop_adapter_set_index(snoop_adapter_t* adapter, int value);

#endif __SNOOPDLL_H__