#include <stdio.h>
#include "SnoopFile.h"
#include "SnoopAdapter.h"

void usage()
{
  printf("Packet Generator version 5.0\n");
  printf("Copyright (c) NetworkLab Corporation All rights reserved\n");
  printf("\n");
  printf("pg [ts] <adapterIndex> <fileName>\n");
  printf("\n");
  printf("  ts : keep time sync\n");
  printf("\n");
  printf("example\n");
  printf("\n");
  printf("  pg 1 test.cap\n");
  printf("\n");

  int count = snoopGetAdapterCount();
  for (int i = 0; i < count; i++)
  {
    printf("%d : %s\n", i, snoopGetAdapterInfo(i)->description);
  }
}

class Param
{
public:
  bool   timeSync;
  int    adapterIndex;
  string fileName;
  bool   parse(int argc, char* argv[]);
};

bool Param::parse(int argc, char* argv[])
{
  timeSync     = false;
  adapterIndex = SNOOP_INVALID_ADAPTER_INDEX;
  fileName     = "";

  int index = 0;
  for (int i = 1; i < argc; i++)
  {
    if (strcmp(argv[i], "ts") == 0)
    {
      timeSync = true;
      continue;
    }
    switch (index)
    {
      case 0 : adapterIndex = atoi(argv[i]); break;
      case 1 : fileName     = argv[i]; break;
      default: printf("extra parameters\n\n"); return false;
    }
    index++;
  }

  if (adapterIndex == SNOOP_INVALID_ADAPTER_INDEX) return false;
  if (fileName == "") return false;
  return true;
}

int main(int argc, char* argv[])
{
  Param param;
  if (!param.parse(argc, argv))
  {
    usage();
    return 0;
  }

  SnoopFile    file;
  SnoopAdapter adapter;

  file.autoCapture     = false;
  file.fileName        = param.fileName;
  file.timeSync        = param.timeSync;

  adapter.autoCapture  = false;
  adapter.adapterIndex = param.adapterIndex;

  if (!file.open())
  {
    printf("can not open file(%s)\n", file.error.msg);
    return 0;
  }
  if (!adapter.open())
  {
    printf("can not open adapter(%s)\n", adapter.error.msg);
    return 0;
  }

  while (true)
  {
    SnoopPacket packet;

    int res = file.read(&packet);
    if (res <= 0) break;
    adapter.write(packet.pktData, packet.pktHdr->caplen);
  }

  file.close();
  adapter.close();

  return 0;
}