#include "stdafx.h"
#include "sc_impl.h"
#include "VDXMLFile.h"
#include "SnoopAdapter.h"
#include "SnoopCaptureFactory.h"

// ----------------------------------------------------------------------------
// SnoopCaptureImpl
// ----------------------------------------------------------------------------
SnoopCaptureImpl::SnoopCaptureImpl(void* param)
{
  m_capture = NULL;
  if (param != NULL)
  {
    load((char*)param);
  }
}

SnoopCaptureImpl::~SnoopCaptureImpl()
{
  SAFE_DELETE(m_capture);
}

void SnoopCaptureImpl::_onCapture(SnoopCapture* capture, SnoopPacket*  packet)
{
  SnoopCaptureImpl* captureImpl = (SnoopCaptureImpl*)capture->owner;
  captureImpl->onCapture(captureImpl, packet->pktHdr, packet->pktData);
}

char* SnoopCaptureImpl::error()
{
  if (m_capture == NULL) return "object is null";
  return m_capture->error.msg;
}

bool SnoopCaptureImpl::open()
{
  if (m_capture == NULL) return false;
  return m_capture->open();
}

bool SnoopCaptureImpl::close()
{
  if (m_capture == NULL) return false;
  return m_capture->close();
}

int SnoopCaptureImpl::write(char* buf, int size)
{
  if (m_capture == NULL) return -1;
  return m_capture->write(buf, size);
}

int SnoopCaptureImpl::captureType()
{
  if (m_capture == NULL) return SCT_NONE;
  return m_capture->captureType();
}

int SnoopCaptureImpl::relay(struct pcap_pkthdr* header, char* packet)
{
  if (m_capture == NULL) return -1;
  SnoopPacket _packet;
  _packet.pktHdr  = header;
  _packet.pktData = packet;
  return m_capture->relay(&_packet);
}

void SnoopCaptureImpl::setCapture(ISnoopCapture_OnCapture onCapture)
{
  this->onCapture = onCapture;
}

bool SnoopCaptureImpl::load(char* xmlFileName)
{
  if (m_capture != NULL)
  {
    delete m_capture;
	m_capture = NULL;
  }
  VDXMLFile xmlFile;
  bool res = xmlFile.loadFromFile(xmlFileName);
  string className = "SnoopAdapter";
  if (res)
	className = xmlFile.root->getStr("class", "SnoopAdapter");
  m_capture = SnoopCaptureFactory::createByClassName(className);
  if (m_capture == NULL)
  {
    LOG_ERROR("[sm_impe.cpp] SnoopCaptureImpl::load m_capture is NULL");
	return false;
  }
  m_capture->load(xmlFile.root);
  m_capture->owner = this;
  m_capture->onCapture = _onCapture;
  return true;
}

bool SnoopCaptureImpl::save(char* xmlFileName)
{
  if (m_capture == NULL) return false;
  VDXMLFile xmlFile;
  xmlFile.root->setValue("SnoopCapture");
  xmlFile.root->setStr("class", m_capture->className());
  m_capture->save(xmlFile.root);
  bool res = xmlFile.saveToFile(xmlFileName);
  return res;
}
