//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FrameSnoopHostList.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TSnoopHostListFrame *SnoopHostListFrame;

//---------------------------------------------------------------------------
// FindHostNameThread
//---------------------------------------------------------------------------
FindHostNameThread::FindHostNameThread(TSnoopHostListFrame* owner)
{
  this->owner = owner;
  ip = "";
  name = "?";
}

FindHostNameThread::~FindHostNameThread()
{
  close();
}

bool FindHostNameThread::open(string ip)
{
  this->ip = ip;
  return VDThread::open();
}

bool FindHostNameThread::close(bool wait)
{
  return VDThread::close(wait);
}

void FindHostNameThread::run()
{
  struct in_addr addr;
  struct hostent* remoteHost;

  setPriority(THREAD_PRIORITY_LOWEST);
  addr.s_addr = inet_addr(ip.c_str());
  if (addr.s_addr == INADDR_NONE)
  {
    LOG_ERROR("[FrameSnoopHostList.cpp] FindHostNameThread::run addr.s_addr is INADDR_NONE");
    synchronize(show, NULL);
    return;
  }
  remoteHost = gethostbyaddr((char *) &addr, 4, AF_INET);
  if (remoteHost == NULL)
  {
    DWORD error = WSAGetLastError();
    switch (error)
    {
      case WSAHOST_NOT_FOUND:
        LOG_ERROR("[FrameSnoopHostList.cpp] FindHostNameThread::run Host not found");
        break;
      case WSANO_DATA:
        // LOG_ERROR("[FrameSnoopHostList.cpp] FindHostNameThread::run No data record found");
        break;
      default:
        LOG_ERROR("[FrameSnoopHostList.cpp] FindHostNameThread::run Function failed with error: %ld", error);
    }
    synchronize(show, NULL);
    return;
  }
  synchronize(show, remoteHost);
}

void FindHostNameThread::show(VDThread* thread, void* user)
{
  FindHostNameThread* aThread = (FindHostNameThread*)thread;
  struct hostent* remoteHost = (struct hostent*)user;

  string hostName = "?";
  if (remoteHost != NULL)
    hostName = remoteHost->h_name;

  TListItem* ListItem = aThread->owner->findListItem(aThread->ip);
  if (ListItem != NULL)
    ListItem->Caption = hostName.c_str();
}

//---------------------------------------------------------------------------
// SendARPRequestAllThread
//---------------------------------------------------------------------------
SendARPRequestAllThread::SendARPRequestAllThread(TSnoopHostListFrame* owner)
{
  this->owner = owner;
}

SendARPRequestAllThread::~SendARPRequestAllThread()
{
  close();
}

void SendARPRequestAllThread::run()
{
  if (!owner->findHost.sendARPRequestAll()) return;
  owner->tmrClose->Enabled = false;
  owner->tmrClose->Enabled = true;
}

//---------------------------------------------------------------------------
void TSnoopHostListFrame::initializeControl()
{
  findHost.owner = this;
  findHost.autoCapture = true;
#ifdef SNOOP_THREAD_SAFE_SUPPORT
  findHost.threadSafe = true;
#endif SNOOP_THREAD_SAFE_SUPPORT
  findHost.scanInterval = 0;
  findHost.sendInterval = 50;
  findHost.onCapture = _findHost_OnCapture;
  threadList.clear();
  sendARPRequestAllThread = NULL;
  addSelf = true;

  lvHostList->Align = alClient;

  tmrClose->Interval = SNOOP_DEFAULT_TIMEOUT;
}

void TSnoopHostListFrame::finalizeControl()
{
  tmrCloseTimer(tmrClose);
  threadList.clear();
}

void TSnoopHostListFrame::loadControl(SnoopHostList* hostList)
{
  lvHostList->Items->Clear();
  for (int i = 0; i < hostList->count(); i++)
  {
    SnoopHost* host = hostList->at(i);
    TListItem* item = lvHostList->Items->Add();
    item->Caption = host->name.c_str();
    item->SubItems->Add(ip_to_str(host->ip).c_str());
    item->SubItems->Add(host->mac.str().c_str());
  }
}

void TSnoopHostListFrame::saveControl(SnoopHostList* hostList)
{
  hostList->clear();
  for (int i = 0 ; i < lvHostList->Items->Count; i++)
  {
    TListItem* item = lvHostList->Items->Item[i];
    SnoopHost* host = hostList->add();
    host->name = item->Caption.c_str();
    host->ip = str_to_ip(item->SubItems->Strings[0].c_str());
    host->mac = item->SubItems->Strings[1].c_str();
  }
}

void TSnoopHostListFrame::setControl()
{
  btnScan->Enabled = !active();
}

bool TSnoopHostListFrame::active()
{
  return findHost.active();
}

void TSnoopHostListFrame::scan(int adapterIndex, IP startIP, IP endIP)
{
  if (findHost.active())
  {
    Application->MessageBox("Still scanning...", "Error", MB_ICONERROR);
    return;
  }
  findHost.adapterIndex = adapterIndex;
  findHost.hostList.clear();
  for (IP ip = startIP; ip <= endIP; ip++)
  {
    findHost.hostList.add(ip);
  }

  if (!findHost.open()) return;

  if (sendARPRequestAllThread == NULL)
  {
    sendARPRequestAllThread = new SendARPRequestAllThread(this);
    sendARPRequestAllThread->open();
  }

  setControl();

  //
  // Add self
  //
  if (addSelf)
  {
    SnoopNetInfo netInfo;
    netInfo.adapterIndex = adapterIndex;
    if (findHost.onCapture != NULL)
      findHost.onCapture(&findHost, &netInfo.ip, &netInfo.mac);
  }
}

void TSnoopHostListFrame::scan(int adapterIndex, string startIP, string endIP)
{
  scan(adapterIndex, str_to_ip(startIP), str_to_ip(endIP));
}

void TSnoopHostListFrame::load(VDXML* xml)
{
  tmrClose->Interval = xml->getInt("closeInterval", tmrClose->Interval);
  addSelf = xml->getBool("addSelf", addSelf);

  //
  // Column
  //
  VDXML* columnXml = xml->gotoChild("Column");
  for (int i = 0; i < lvHostList->Columns->Count; i++)
  {
    TListColumn* ListColumn = lvHostList->Columns->Items[i];
    ListColumn->Width = columnXml->getInt("width" + int_to_str(i), ListColumn->Width);
  }

  //
  // HostList
  //
  VDXML* hostListXml = xml->gotoChild("HostList");
  lvHostList->Items->Clear();
  for (int i = 0; i < hostListXml->getChildCount(); i++)
  {
    VDXML* childXml = hostListXml->getChild(i);
    TListItem* ListItem = lvHostList->Items->Add();

    ListItem->Checked = childXml->getBool("checked", false);
    ListItem->Caption = childXml->getStr("name", "").c_str();
    ListItem->SubItems->Add(childXml->getStr("ip", "").c_str());
    ListItem->SubItems->Add(childXml->getStr("mac", "").c_str());
  }

  //
  // FindHost
  //
  VDXML* findHostXml = xml->gotoChild("FindHost");
  findHost.load(findHostXml);
}

void TSnoopHostListFrame::save(VDXML* xml)
{
  xml->setInt("closeInterval", tmrClose->Interval);
  xml->setBool("addSelf", addSelf);

  //
  // Column
  //
  VDXML* columnXml = xml->gotoChild("Column");
  for (int i = 0; i < lvHostList->Columns->Count; i++)
  {
    TListColumn* ListColumn = lvHostList->Columns->Items[i];
    columnXml->setInt("width" + int_to_str(i), ListColumn->Width);
  }

  //
  // HostList
  //
  VDXML* hostListXml = xml->gotoChild("HostList");
  hostListXml->clearChild();
  for (int i = 0; i < lvHostList->Items->Count; i++)
  {
    TListItem* ListItem = lvHostList->Items->Item[i];
    VDXML* childXml = hostListXml->addChild("Host");

    childXml->setBool("checked", ListItem->Checked);
    childXml->setStr("name", ListItem->Caption.c_str());
    childXml->setStr("ip", ListItem->SubItems->Strings[0].c_str());
    childXml->setStr("mac", ListItem->SubItems->Strings[1].c_str());
  };

  //
  // FindHost
  //
  VDXML* findHostXml = xml->gotoChild("FindHost");
  findHost.save(findHostXml);
}

void TSnoopHostListFrame::_findHost_OnCapture(
  SnoopFindHost* snoopFindHost,
  IP*            ip,
  MAC*           mac)
{
  TSnoopHostListFrame* frame = (TSnoopHostListFrame*)snoopFindHost->owner;
  frame->findHost_OnCapture(snoopFindHost, ip, mac);
}

void TSnoopHostListFrame::findHost_OnCapture(
  SnoopFindHost* snoopFindHost,
  IP*            ip,
  MAC*           mac)
{
  string s = *mac;
  TListItem* ListItem = findListItem(ip_to_str(*ip));

  if (ListItem == NULL)
  {
    ListItem = lvHostList->Items->Add();
    ListItem->Caption = "";
    ListItem->SubItems->Clear();
    ListItem->SubItems->Add(ip_to_str(*ip).c_str());
    ListItem->SubItems->Add((*mac).str().c_str());
  } else
  {
    ListItem->SubItems->Strings[1] = (*mac).str().c_str();
  }

  if (ListItem->Caption == "")
  {
    FindHostNameThread* thread = new FindHostNameThread(this);
    threadList.add(thread);
    thread->open(ip_to_str(*ip));
  }

  if (findHost.isFoundAll())
  {
    // can not findHost.close() directly, so call PostMessage.
    PostMessage(Handle, WM_MYCLOSE, 0, 0);
  }
}

TListItem* TSnoopHostListFrame::findListItem(string ip)
{
  for (int i = 0; i < lvHostList->Items->Count; i++)
  {
    TListItem* ListItem = lvHostList->Items->Item[i];
    if (ListItem->SubItems->Strings[0].c_str() == ip)
      return ListItem;
  }
  return NULL;
}

//---------------------------------------------------------------------------
__fastcall TSnoopHostListFrame::TSnoopHostListFrame(TComponent* Owner)
  : TFrame(Owner)
{
  initializeControl();
}
//---------------------------------------------------------------------------
__fastcall TSnoopHostListFrame::~TSnoopHostListFrame()
{
  finalizeControl();
}
//---------------------------------------------------------------------------
void __fastcall TSnoopHostListFrame::Loaded()
{
  ParentColor = true;
}

void __fastcall TSnoopHostListFrame::Dispatch(void *Message)
{ 
  TMessage *msg = (PMessage)Message;
  switch (msg->Msg)
  {
    case WM_MYCLOSE:
      tmrCloseTimer(tmrClose);
      tmrClose->OnTimer(tmrClose);
      break;
    default :
      TFrame::Dispatch(Message);
  }
}
//---------------------------------------------------------------------------
void __fastcall TSnoopHostListFrame::tmrCloseTimer(TObject *Sender)
{
  tmrClose->Enabled = false;
  findHost.close();
  SAFE_DELETE(sendARPRequestAllThread);
  setControl();
}
//---------------------------------------------------------------------------
void __fastcall TSnoopHostListFrame::btnScanClick(TObject *Sender)
{
  // scan(adapterIndex, startIP, endIP);
}
//---------------------------------------------------------------------------

void __fastcall TSnoopHostListFrame::btnClearClick(TObject *Sender)
{
  lvHostList->Items->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TSnoopHostListFrame::miSelectAllClick(TObject *Sender)
{
  for (int i = 0; i < lvHostList->Items->Count; i++)
  {
    TListItem* ListItem = lvHostList->Items->Item[i];
    ListItem->Checked = true;
  }
}
//---------------------------------------------------------------------------

void __fastcall TSnoopHostListFrame::miSelectNoneClick(TObject *Sender)
{
  for (int i = 0; i < lvHostList->Items->Count; i++)
  {
    TListItem* ListItem = lvHostList->Items->Item[i];
    ListItem->Checked = false;
  }
}
//---------------------------------------------------------------------------

void __fastcall TSnoopHostListFrame::miSelectReverseClick(TObject *Sender)
{
  for (int i = 0; i < lvHostList->Items->Count; i++)
  {
    TListItem* ListItem = lvHostList->Items->Item[i];
    ListItem->Checked = !ListItem->Checked;
  }
}
//---------------------------------------------------------------------------

void __fastcall TSnoopHostListFrame::lvHostListColumnClick(TObject *Sender,
      TListColumn *Column)
{
  // Tag is used in column discriminator
  // 1:Name / 2:IP / 3:Mac
  // plus: ASC / minus: DESC

  int oldTag = lvHostList->Tag;
  int newTag = Column->Index + 1;
  if (abs(oldTag) == newTag) newTag = -oldTag;
  lvHostList->Tag = newTag;
  lvHostList->AlphaSort();
}
//---------------------------------------------------------------------------

void __fastcall TSnoopHostListFrame::lvHostListCompare(TObject *Sender,
      TListItem *Item1, TListItem *Item2, int Data, int &Compare)
{
  switch (abs(lvHostList->Tag))
  {
    case 1: // Name
      if (Item1->Caption < Item2->Caption) Compare = -1;
      else if (Item1->Caption == Item2->Caption) Compare = 0;
      else Compare = 1;
      break;
    case 2: // IP
      Compare =
        str_to_ip(Item1->SubItems->Strings[0].c_str()) -
        str_to_ip(Item2->SubItems->Strings[0].c_str());
      break;
    case 3: // Mac
      if (Item1->SubItems->Strings[1] < Item2->SubItems->Strings[1]) Compare = -1;
      else if (Item1->SubItems->Strings[1] == Item2->SubItems->Strings[1]) Compare = 0;
      else Compare = 1;
      break;
  default:
    ;
  }
  if (lvHostList->Tag < 0 )
  {
    Compare = -Compare;
  }
}
//---------------------------------------------------------------------------

