//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FrameSnoopSessionList.h"
#include "FormSnoopSession.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "FrameSnoopHostList"
#pragma link "FrameSnoopHostList"
#pragma link "FrameSnoopHostList"
#pragma resource "*.dfm"
TSnoopSessionListFrame *SnoopSessionListFrame;
//---------------------------------------------------------------------------
void TSnoopSessionListFrame::initializeControl()
{
  pcSessionList->ActivePageIndex = 0;
  pnlMain->Align = alClient;
  pcSessionList->Align = alClient;
  SnoopHostListFrame1->Align = alClient;
  lvSessionList->Align = alClient;
}

void TSnoopSessionListFrame::finalizeControl()
{
}

void TSnoopSessionListFrame::loadControl(SnoopARPSpoofSessionList* sessionList, string gateway)
{
  int i;
  int direction = cbxDirection->ItemIndex;
  TListItem* ListItem;

  sessionList->clear();
  switch (pcSessionList->ActivePageIndex)
  {
    case 0: // Basic
      for (i = 0; i < SnoopHostListFrame1->lvHostList->Items->Count; i++)
      {
        ListItem = SnoopHostListFrame1->lvHostList->Items->Item[i];
        if (!ListItem->Checked) continue;

        string ip = ListItem->SubItems->Strings[0].c_str();

        if (direction == 0 || direction == 2) // ->GW, <-> GW
        {
          sessionList->add(ip, gateway);
        }

        if (direction == 1 || direction == 2) // <-GW, <-> GW
        {
          sessionList->add(gateway, ip);
        }
      }
      break;
    case 1: // Custom
      for (i = 0; i < lvSessionList->Items->Count; i++)
      {
        ListItem = lvSessionList->Items->Item[i];
        if (!ListItem->Checked) continue;
        
        string sender = ListItem->Caption.c_str();
        string target = ListItem->SubItems->Strings[0].c_str();

        sessionList->add(sender, target);
      }
      break;
  }
}

void TSnoopSessionListFrame::saveControl(SnoopARPSpoofSessionList* sessionList, string gateway)
{
  LOG_ERROR("[FrameSnoopSessionList.cpp] TSnoopSessionListFrame::saveControl not supported");
}

void TSnoopSessionListFrame::setControl()
{
  bool selected = lvSessionList->Selected != NULL;
  btnDelete->Enabled = selected;
  btnEdit->Enabled = selected;
}

void TSnoopSessionListFrame::load(VDXML* xml)
{
  pcSessionList->ActivePageIndex = xml->getInt("activePageIndex", pcSessionList->ActivePageIndex);
  cbxDirection->ItemIndex = xml->getInt("direction", cbxDirection->ItemIndex);

  //
  // Basic
  //
  SnoopHostListFrame1->load(xml->gotoChild("Basic"));

  //
  // Custom
  //

  VDXML* customXml = xml->gotoChild("Custom");

    //
    // Column
    //
    VDXML* columnXml = customXml->gotoChild("Column");
    for (int i = 0; i < lvSessionList->Columns->Count; i++)
    {
      TListColumn* ListColumn = lvSessionList->Columns->Items[i];
      ListColumn->Width = columnXml->getInt("width" + int_to_str(i), ListColumn->Width);
    }

    //
    // SessionList
    //
    VDXML* sessionListXml = customXml->gotoChild("SessionList");
    lvSessionList->Items->Clear();
    for (int i = 0; i < sessionListXml->getChildCount(); i++)
    {
      VDXML* childXml = sessionListXml->getChild(i);
      TListItem* ListItem = lvSessionList->Items->Add();

      ListItem->Checked = childXml->getBool("checked", false);
      ListItem->Caption = childXml->getStr("sender", "").c_str();
      ListItem->SubItems->Add(childXml->getStr("target", "").c_str());
    }
}

void TSnoopSessionListFrame::save(VDXML* xml)
{
  xml->setInt("activePageIndex", pcSessionList->ActivePageIndex);
  xml->setInt("direction", cbxDirection->ItemIndex);

  //
  // Basic
  //
  SnoopHostListFrame1->save(xml->gotoChild("Basic"));

  //
  // Custom
  //

  VDXML* customXml = xml->gotoChild("Custom");

    //
    // Column
    //
    VDXML* columnXml = customXml->gotoChild("Column");
    for (int i = 0; i < lvSessionList->Columns->Count; i++)
    {
      TListColumn* ListColumn = lvSessionList->Columns->Items[i];
      columnXml->setInt("width" + int_to_str(i), ListColumn->Width);
    }

    //
    // SessionList
    //
    VDXML* sessionListXml = customXml->gotoChild("SessionList");
    sessionListXml->clearChild();
    for (int i = 0; i < lvSessionList->Items->Count; i++)
    {
      TListItem* ListItem = lvSessionList->Items->Item[i];
      VDXML* childXml = sessionListXml->addChild("session");

      childXml->setBool("checked", ListItem->Checked);
      childXml->setStr("sender", ListItem->Caption.c_str());
      childXml->setStr("target", ListItem->SubItems->Strings[0].c_str());
    }
}

//---------------------------------------------------------------------------
__fastcall TSnoopSessionListFrame::TSnoopSessionListFrame(TComponent* Owner)
  : TFrame(Owner)
{
  initializeControl();
  setControl();
}
//---------------------------------------------------------------------------
__fastcall TSnoopSessionListFrame::~TSnoopSessionListFrame()
{
  finalizeControl();
}
//---------------------------------------------------------------------------
void __fastcall TSnoopSessionListFrame::Loaded()
{
  ParentColor = true;
}
//---------------------------------------------------------------------------
void __fastcall TSnoopSessionListFrame::btnAddClick(TObject *Sender)
{
  if (SnoopSessionForm == NULL)
    SnoopSessionForm = new TSnoopSessionForm(Application);

  SnoopSessionForm->edSender->Text = "";
  SnoopSessionForm->edTarget->Text = "";
  SnoopSessionForm->ShowModal();
  if (SnoopSessionForm->Tag == mrOk)
  {
    TListItem* ListItem = lvSessionList->Items->Add();

    ListItem->Checked = true;
    ListItem->Caption = SnoopSessionForm->edSender->Text;
    ListItem->SubItems->Add(SnoopSessionForm->edTarget->Text);
  }
  setControl();
}
//---------------------------------------------------------------------------

void __fastcall TSnoopSessionListFrame::btnDeleteClick(TObject *Sender)
{
  TListItem* ListItem = lvSessionList->Selected;
  if (ListItem == NULL) return;
  lvSessionList->Items->Delete(ListItem->Index);
  setControl(); 
}
//---------------------------------------------------------------------------

void __fastcall TSnoopSessionListFrame::btnEditClick(TObject *Sender)
{
  TListItem* ListItem = lvSessionList->Selected;
  if (ListItem == NULL) return;

  if (SnoopSessionForm == NULL)
    SnoopSessionForm = new TSnoopSessionForm(Application);

  SnoopSessionForm->edSender->Text = ListItem->Caption;
  SnoopSessionForm->edTarget->Text = ListItem->SubItems->Strings[0];
  SnoopSessionForm->ShowModal();
  if (SnoopSessionForm->Tag == mrOk)
  {

    ListItem->Checked = true;
    ListItem->Caption = SnoopSessionForm->edSender->Text;
    ListItem->SubItems->Strings[0] = SnoopSessionForm->edTarget->Text;
  }
  setControl();
}
//---------------------------------------------------------------------------

void __fastcall TSnoopSessionListFrame::lvSessionListClick(TObject *Sender)
{
  setControl();  
}
//---------------------------------------------------------------------------


void __fastcall TSnoopSessionListFrame::lvSessionListKeyDown(TObject *Sender,
      WORD &Key, TShiftState Shift)
{
  LOG_DEBUG("key=%d\n", Key);
  if (Key == 46)
  { // Del Key
    btnDelete->Click();
  }
}
//---------------------------------------------------------------------------

void __fastcall TSnoopSessionListFrame::lvSessionListDblClick(TObject *Sender)
{
  btnEdit->Click();  
}
//---------------------------------------------------------------------------

void __fastcall TSnoopSessionListFrame::miSelectAllClick(TObject *Sender)
{
  for (int i = 0; i < lvSessionList->Items->Count; i++)
  {
    TListItem* ListItem = lvSessionList->Items->Item[i];
    ListItem->Checked = true;
  }
}
//---------------------------------------------------------------------------

void __fastcall TSnoopSessionListFrame::miSelectNoneClick(TObject *Sender)
{
  for (int i = 0; i < lvSessionList->Items->Count; i++)
  {
    TListItem* ListItem = lvSessionList->Items->Item[i];
    ListItem->Checked = false;
  }
}
//---------------------------------------------------------------------------

void __fastcall TSnoopSessionListFrame::miSelectReverseClick(TObject *Sender)
{
  for (int i = 0; i < lvSessionList->Items->Count; i++)
  {
    TListItem* ListItem = lvSessionList->Items->Item[i];
    ListItem->Checked = !ListItem->Checked;
  }
}
//---------------------------------------------------------------------------

void __fastcall TSnoopSessionListFrame::SnoopHostListFrame1btnScanClick(
      TObject *Sender)
{
  SnoopHostListFrame1->btnScanClick(Sender);
}
//---------------------------------------------------------------------------

