#include "SnoopARPSpoof.h"
#include "SnoopTCP.h" // for SnoopTCP::checksum

REGISTER_METACLASS(SnoopARPSpoof, SnoopCapture)

// ----------------------------------------------------------------------------
// SnoopARPSpoofSession
// ----------------------------------------------------------------------------
SnoopARPSpoofSession::SnoopARPSpoofSession()
{
  clear();
}

SnoopARPSpoofSession::~SnoopARPSpoofSession()
{
  clear();
}

void SnoopARPSpoofSession::clear()
{
  senderIP     = 0;
  senderMac    = snoopCleanMac;
  targetIP     = 0;
  targetMac    = snoopCleanMac;
}

void SnoopARPSpoofSession::load(VDXML* xml)
{
  senderIP     = str_to_ip(xml->getStr("senderIP", ip_to_str(senderIP)));
  targetIP     = str_to_ip(xml->getStr("targetIP", ip_to_str(targetIP)));
}

void SnoopARPSpoofSession::save(VDXML* xml)
{
  xml->setStr("senderIP", ip_to_str(senderIP));
  xml->setStr("targetIP", ip_to_str(targetIP));
}

// ----------------------------------------------------------------------------
// SnoopARPSpoofSessionList
// ----------------------------------------------------------------------------
SnoopARPSpoofSessionList::SnoopARPSpoofSessionList()
{
  owner = NULL;
  clear();
}

SnoopARPSpoofSessionList::~SnoopARPSpoofSessionList()
{
  clear();
}

SnoopARPSpoofSession* SnoopARPSpoofSessionList::add(IP senderIP, IP targetIP, MAC senderMac, MAC targetMac)
{
  SnoopARPSpoofSession* res = NULL;
  SnoopARPSpoofSession* session;
  lock();

  if (find(senderIP, targetIP) != NULL) goto _end; // if already exists.

  session = new SnoopARPSpoofSession;
  session->senderIP = senderIP;
  session->targetIP = targetIP;
  session->senderMac = senderMac;
  session->targetMac = targetMac;

  //
  // If owner(SnoopARPSpoof or SnoopARPRecover) is active, MAC value must not be clear.
  //
  if (owner != NULL && owner->active())
  {
    bool ok = true;
	if (senderMac.isClear())
	{
      LOG_ERROR("[SnoopARPSpoof.cpp] ****************************************************");
	  LOG_ERROR("[SnoopARPSpoof.cpp] SnoopARPSpoofSessionList::add senderMac(%s) is clear", targetMac.str().c_str());
	  LOG_ERROR("[SnoopARPSpoof.cpp] ****************************************************");
	  ok = false;
	}
	if (targetMac.isClear())
	{
      LOG_ERROR("[SnoopARPSpoof.cpp] ****************************************************");
	  LOG_ERROR("[SnoopARPSpoof.cpp] SnoopARPSpoofSessionList::add targetMac(%s) is clear", targetMac.str().c_str());
	  LOG_ERROR("[SnoopARPSpoof.cpp] ****************************************************");
	  ok = false;
	}
	if (!ok)
	{
      delete session;
	  goto _end;
	}

    if (IS_CLASS(owner, SnoopARPSpoof*))
	  owner->sendARPInfect(session);
	if (IS_CLASS(owner, SnoopARPRecover*))
      owner->sendARPInfect(session);
  }

  if (VDLockPList<SnoopARPSpoofSession*>::add(session) == -1)
  {
    LOG_ERROR("[SnoopARPSpoof.cpp] SnoopARPSpoofSessionList::add add return -1");
	delete session;
	goto _end;
  }

  res = session;

_end:
  unlock();
  return res;
}

SnoopARPSpoofSession* SnoopARPSpoofSessionList::add(string senderIP, string targetIP, MAC senderMac, MAC targetMac)
{
  return add(str_to_ip(senderIP), str_to_ip(targetIP), senderMac, targetMac);
}

SnoopARPSpoofSession* SnoopARPSpoofSessionList::_del(IP senderIP, IP targetIP)
{
  SnoopARPSpoofSession* res = NULL;

  lock();
  SnoopARPSpoofSession* session = find(senderIP, targetIP);
  if (session == NULL) goto _end;
  if (owner != NULL && owner->active())
  {
    if (IS_CLASS(owner, SnoopARPSpoof*))
      owner->sendARPRecover(session);
	if (IS_CLASS(owner, SnoopARPRecover*))
      owner->sendARPInfect(session);
  }
  if (VDLockPList<SnoopARPSpoofSession*>::remove(session) == -1)
  {
    LOG_ERROR("[SnoopARPSpoof.cpp] SnoopARPSpoofSessionList::del remove return -1");
	goto _end;
  }
  res = session;
_end:
  unlock();
  return res;
}

SnoopARPSpoofSession* SnoopARPSpoofSessionList::_del(string senderIP, string targetIP)
{
  return _del(str_to_ip(senderIP), str_to_ip(targetIP));
}

SnoopARPSpoofSession* SnoopARPSpoofSessionList::find(IP senderIP, IP targetIP)
{
  SnoopARPSpoofSession* res;
  for (int i = 0; i < count(); i++)
  {
    res = at(i);
    if (senderIP == res->senderIP && targetIP == res->targetIP)
      return res;
  }
  return NULL;
}

void SnoopARPSpoofSessionList::load(VDXML* xml)
{
  int count = xml->getChildCount();
  clear();
  for (int i = 0; i < count; i++)
  {
    VDXML* childXML = xml->getChild(i);
    SnoopARPSpoofSession* session = this->add(0, 0);
    session->load(childXML);
  }
}

void SnoopARPSpoofSessionList::save(VDXML* xml)
{
  int _count = count();
  xml->clearChild();
  for (int i = 0; i < _count; i++)
  {
    SnoopARPSpoofSession* session = this->at(i);
    VDXML* childXML = xml->addChild("Session");
    session->save(childXML);
  }
}

// ----------------------------------------------------------------------------
// SnoopARPSpoofInfectThread
// ----------------------------------------------------------------------------
SnoopARPSpoofInfectThread::SnoopARPSpoofInfectThread(SnoopARPSpoof *owner) : VDThread((void*)owner)
{
  event = NULL;
}

SnoopARPSpoofInfectThread::~SnoopARPSpoofInfectThread()
{
  close();
  SAFE_DELETE(event);
}

bool SnoopARPSpoofInfectThread::open()
{
  if (event == NULL)
    event = new VDEvent(NULL, false, false, NULL);
  return VDThread::open();
}

bool SnoopARPSpoofInfectThread::close()
{
  bool res;
  if (event != NULL)
    event->setEvent();
  res = VDThread::close(true);
  SAFE_DELETE(event);
  return res;
}

void SnoopARPSpoofInfectThread::run()
{
  // lock(); unlock(); // Wait until open() is completed. // gilgil temp 2009.08.31

  SnoopARPSpoof* arpSpoof = (SnoopARPSpoof*)owner;
  DWORD res;

  while (active())
  {
    res = event->waitFor((DWORD)arpSpoof->infectInterval);
    if (res != WAIT_TIMEOUT) break;
	arpSpoof->sessionList.lock();
    arpSpoof->sendARPInfectAll();
	arpSpoof->sessionList.unlock();
  }
}

// ----------------------------------------------------------------------------
// _SnoopARPSpoof
// ----------------------------------------------------------------------------
_SnoopARPSpoof::_SnoopARPSpoof(void* owner) : SnoopAdapter(owner)
{
  findHost.scanInterval = 1000;
  findHost.sendInterval = 0;
  virtualMac = snoopCleanMac;
  realVirtualMac = snoopCleanMac;
  selfRelay = true;
  sessionList.owner = this;
}

_SnoopARPSpoof::~_SnoopARPSpoof()
{
  close();
}

bool _SnoopARPSpoof::doOpen()
{
  //
  // Inherited open
  //
  string tempFilter = filter;
  filter = "arp or ip";
  if (!SnoopAdapter::doOpen()) goto _error;
  filter = tempFilter;

  //
  // Set realVirtualMac using virtualMac
  //
  if (virtualMac.isClear())
    realVirtualMac = netInfo.mac;
  else
    realVirtualMac = virtualMac;
  LOG_DEBUG("[SnoopARPSpoof.cpp] _SnoopARPSpoof::open realVirtualMac=%s", realVirtualMac.str().c_str());

  return true;
_error:
  SnoopAdapter::doClose();
  return false;
}

bool _SnoopARPSpoof::doClose()
{
  return SnoopAdapter::doClose();
}

int _SnoopARPSpoof::relay(SnoopPacket* packet)
{
  return relay(packet->pktHdr, packet->ethHdr, packet->ipHdr);
}

int _SnoopARPSpoof::relay(PKT_HDR* pktHdr, ETH_HDR* ethHdr, IP_HDR* ipHdr)
{
  // ----- by gilgil 2007.06.12 -----
  TCP_HDR* tcpHdr;
  // Strange to say, TCP checksum can not be correct(Self Spoofing Windows XP).
  // if (SnoopIP::isTCP(ipHdr, &tcpHdr) && ((tcpHdr->rsvd_flags & TCP_FLAG_SYN) != 0)) // gilgil temp 2009.09.01
  if (SnoopIP::isTCP(ipHdr, &tcpHdr) && ((tcpHdr->th_flags & TH_SYN) != 0))
  {
    UINT16 oldChecksum = ntohs(tcpHdr->th_sum);
    UINT16 newChecksum = SnoopTCP::checksum(ipHdr, tcpHdr); // gilgil temp 2008.10.20
    tcpHdr->th_sum = htons(newChecksum);
    LOG_DEBUG("[SnoopARPSpoof.cpp] SnoopARPSpoof::relay checksum 0x%x > 0x%x", oldChecksum, newChecksum); // gilgil temp 2008.10.22
  }
  // --------------------------------
  ethHdr->ether_shost = realVirtualMac;
  return write((char*)ethHdr, pktHdr->caplen);
}

void _SnoopARPSpoof::setAdapterIndex(int value)
{
  SnoopAdapter::setAdapterIndex(value);
  findHost.setAdapterIndex(value);
  netInfo.setAdapterIndex(value);
}

int _SnoopARPSpoof::sendARPInfect(int index)
{
  SnoopARPSpoofSession* session = sessionList.at(index);
  if (session == NULL) -1;
  return sendARPInfect(session);
}

int _SnoopARPSpoof::sendARPInfect(SnoopARPSpoofSession* session)
{
  ETH_HDR ethHdr;
  ARP_HDR arpHdr;

  //
  // Set ethHdr
  //
  ethHdr.ether_dhost = session->senderMac;
  ethHdr.ether_shost = this->realVirtualMac;
  ethHdr.ether_type  = htons(ETHERTYPE_ARP);

  //
  // Set arpHdr
  //
  arpHdr.ar_hrd = htons(1);
  arpHdr.ar_pro = htons(2048);
  arpHdr.ar_hln = 6;
  arpHdr.ar_pln = 4;
  arpHdr.ar_op  = htons(ARPOP_REPLY);
  arpHdr.ar_sa  = this->realVirtualMac;
  arpHdr.ar_si  = htonl(session->targetIP);
  arpHdr.ar_ta  = session->senderMac;
  arpHdr.ar_ti  = htonl(session->senderIP);

  // send ARP packet
  int res = snoopSendARP(m_pcap, &ethHdr, &arpHdr);
  if (res < 0)
  {
    LOG_ERROR("[SnoopARPSpoof.cpp] ****************************************************");
    LOG_ERROR("[SnoopARPSpoof.cpp] SnoopARPSpoof::sendARPInfect sendARPInfect return %d", res);
    LOG_ERROR("[SnoopARPSpoof.cpp] ****************************************************");
  }
  return res;
}

int _SnoopARPSpoof::sendARPInfectAll()
{
  for (int i = 0; i < sessionList.count(); i++)
    if (sendARPInfect(i) == -1) return -1;
  return 1;
}

int _SnoopARPSpoof::sendARPRecover(int index)
{
  SnoopARPSpoofSession* session = sessionList.at(index);
  if (session == NULL) -1;
  return sendARPRecover(session);
}

int _SnoopARPSpoof::sendARPRecover(SnoopARPSpoofSession* session)
{
  ETH_HDR ethHdr;
  ARP_HDR arpHdr;

  //
  // Set ethHdr
  //
  ethHdr.ether_dhost = session->senderMac;
  ethHdr.ether_shost = this->realVirtualMac;
  ethHdr.ether_type  = htons(ETHERTYPE_ARP);

  //
  // Set arpHdr
  //
  arpHdr.ar_hrd = htons(1);
  arpHdr.ar_pro = htons(2048);
  arpHdr.ar_hln = 6;
  arpHdr.ar_pln = 4;
  arpHdr.ar_op = htons(ARPOP_REPLY);
  arpHdr.ar_sa = session->targetMac;
  arpHdr.ar_si = htonl(session->targetIP);
  arpHdr.ar_ta = session->senderMac;
  arpHdr.ar_ti = htonl(session->senderIP);

  // send ARP packet
  int res = snoopSendARP(m_pcap, &ethHdr, &arpHdr);
  if (res < 0)
  {
    LOG_ERROR("[SnoopARPSpoof.cpp] ******************************************************");
    LOG_ERROR("[SnoopARPSpoof.cpp] SnoopARPSpoof::sendARPRecover sendARPRecover return %d", res);
    LOG_ERROR("[SnoopARPSpoof.cpp] *****************************************************");
  }
  return res;
}

int _SnoopARPSpoof::sendARPRecoverAll()
{
  for (int i = 0; i < sessionList.count(); i++)
    if (sendARPRecover(i) == -1) return -1;
  return 1;
}

bool _SnoopARPSpoof::retrieveSessionList()
{
  bool res;
  VDError findHostError;

  //
  // Check if sessionCount is zero.
  //
  if (sessionList.count() == 0)
  {
    SET_ERROR(SnoopError, "session count is zero", VDERR_UNKNOWN);
    goto _error;
  }

  //
  // Check if source ip is same with target ip
  //
  for (int i = 0; i < sessionList.count(); i++)
  {
    SnoopARPSpoofSession* session = sessionList.at(i);
    if ((session->senderIP == session->targetIP))
    {
      SET_ERROR(SnoopError, format("source ip is same as target ip(%s)", ip_to_str(session->senderIP).c_str()).c_str(), VDERR_UNKNOWN);
      goto _error;
    }
  }

  //
  // Find all ip and mac
  //
  if (!findHost.open()) goto _error;
  findHost.hostList.clear();
  findHost.hostList.add(netInfo.ip, netInfo.mac);
  for (int i = 0; i < sessionList.count(); i++)
  {
    SnoopARPSpoofSession* session = sessionList.at(i);
    findHost.hostList.add(session->senderIP);
    findHost.hostList.add(session->targetIP);
  }
  res = findHost.scan();
  findHostError = findHost.error;
  findHost.close();
  if (!res)
  {
    error = findHostError;
    goto _error;
  }

  //
  // Set mac of sessionList from findHost.
  //
  for (int i = 0; i < sessionList.count(); i++)
  {
    SnoopARPSpoofSession* session = sessionList.at(i);
    SnoopHost* host;

    // Set senderMac according to senderIP
    host = findHost.hostList.findByIP(session->senderIP);
    if (host == NULL)
    {
      SET_ERROR(SnoopError, format("can not find host(%s)", ip_to_str(session->senderIP)).c_str(), VDERR_UNKNOWN);
      goto _error;
    }
    session->senderMac = host->mac;

    // Set targetMac according to targetIP
    host = findHost.hostList.findByIP(session->targetIP);
    if (host == NULL)
    {
      SET_ERROR(SnoopError,
        format("snoop error : can not find host(%s)", ip_to_str(session->targetIP)).c_str(),
        VDERR_UNKNOWN);
      goto _error;
    }
    session->targetMac = host->mac;

    // Check if targetMac is same as realVirtualMac
    if (realVirtualMac == session->targetMac)
    {
      SET_ERROR(SnoopError,
        format("real virtual mac(%s) is same as target mac(IP=%s)",
          realVirtualMac.str().c_str(),
          ip_to_str(session->targetIP).c_str()
        ).c_str(),
        VDERR_UNKNOWN);
      goto _error;
    }
  }

  return true;
_error:
  return false;
}

void _SnoopARPSpoof::preventARPRecover(ETH_HDR* ethHdr, ARP_HDR* arpHdr)
{
  SnoopARPSpoofSession* session;
  UINT16 operation = ntohs(arpHdr->ar_op);
  MAC    senderHA  = arpHdr->ar_sa;
  IP     senderIP  = ntohl(arpHdr->ar_si);
  MAC    targetHA  = arpHdr->ar_ta;
  IP     targetIP  = htonl(arpHdr->ar_ti);

  for (int i = 0; i < sessionList.count(); i++)
  {
    session = sessionList.at(i);

    //
    // If ARP packet is my own infect packet, do not anything
    //
    if (
      (MAC)ethHdr->ether_shost == realVirtualMac &&
      (MAC)ethHdr->ether_dhost == session->senderMac &&

      operation == ARPOP_REPLY &&
      senderHA == this->realVirtualMac &&
      senderIP == session->targetIP &&
      targetHA == session->senderMac &&
      targetIP == session->senderIP)
    {
      continue;
    }

    //
    // If ARP packet recover SnoopARPSpoofSession, send ARP infect packet.
    //
    if (
      (MAC)ethHdr->ether_shost == session->senderMac &&
      // Do not check ethHdr->dst

      // Do not check operation
      senderHA == session->senderMac &&
      // Do not check senderIP
      // Do not check targetHA
      targetIP == session->targetIP)
    {
      sendARPInfect(session);
      LOG_DEBUG("[SnoopARPSpoof.cpp] SnoopARPSpoof::preventARPRecover send ARP infect(%s > %s)", 
        ip_to_str(session->senderIP).c_str(), ip_to_str(session->targetIP).c_str());
    }
  }
}

SnoopARPSpoofSession* _SnoopARPSpoof::findSessionByIPPacket(PKT_HDR* pktHdr, ETH_HDR* ethHdr, IP_HDR* ipHdr)
{
  //
  // If broadcast IP Packet?
  //
  if ((MAC)ethHdr->ether_dhost == snoopBroadcastMac) return NULL;

  //
  // Find session
  //
  IP adjSrcIP = netInfo.getAdjIP(ntohl(ipHdr->ip_src));
  IP adjDstIP = netInfo.getAdjIP(ntohl(ipHdr->ip_dst));
  for (int i = 0; i < sessionList.count(); i++)
  {
    SnoopARPSpoofSession* session = sessionList.at(i);

    if (
      (MAC)ethHdr->ether_shost == session->senderMac &&
      adjSrcIP                 == session->senderIP &&
      (MAC)ethHdr->ether_dhost == realVirtualMac &&
      adjDstIP                 == session->targetIP)
    {
      return session;
    }
  }

  //
  // Process self relay
  //
  if (selfRelay)
  {
    if (
      (MAC)ethHdr->ether_shost    == netInfo.mac &&
      (MAC)ethHdr->ether_dhost    == realVirtualMac &&
      ntohl(ipHdr->ip_src) == netInfo.ip)
    {
      SnoopHost* host = this->findHost.hostList.findByIP(adjDstIP);
      if (host == NULL)
      {
        LOG_ERROR("[SnoopARPSpoof.cpp] ********************************************************************");
        LOG_ERROR("[SnoopARPSpoof.cpp] SnoopARPSpoof::findSessionByIPPacket self relay can not find mac(%s)", ip_to_str(adjDstIP).c_str());
        LOG_ERROR("[SnoopARPSpoof.cpp] ********************************************************************");
        return false;
      }
      ethHdr->ether_dhost = host->mac;
      relay(pktHdr, ethHdr, ipHdr); // gilgil temp 2008.10.22
      // LOG_DEBUG("[SnoopARPSpoof.cpp] SnoopARPSpoof::findSessionByIPPacket self relay"); // gilgil temp 2008.10.22
    }
  }
  return NULL;
}

void _SnoopARPSpoof::load(VDXML* xml)
{
  SnoopAdapter::load(xml);
  virtualMac = xml->getStr("virtualMac", virtualMac);
  selfRelay = xml->getBool("selfRelay", selfRelay);
  VDXML* sessionListXML = xml->gotoChild("SessionList");
  sessionList.load(sessionListXML);
}

void _SnoopARPSpoof::save(VDXML* xml)
{
  SnoopAdapter::save(xml);     

  xml->setStr("virtualMac", virtualMac);
  xml->setBool("selfRelay", selfRelay);
  VDXML* sessionListXML = xml->gotoChild("SessionList");
  sessionList.save(sessionListXML);
}

// ----------------------------------------------------------------------------
// SnoopARPSpoof
// ----------------------------------------------------------------------------
SnoopARPSpoof::SnoopARPSpoof(void* owner) : _SnoopARPSpoof(owner)
{
  recover        = NULL;
  infectThread   = NULL;
  infectInterval = 1000;  // 1 sec
}

SnoopARPSpoof::~SnoopARPSpoof()
{
  close();
  SAFE_DELETE(recover);
}

void SnoopARPSpoof::run()
{
  SnoopPacket packet;

  while (thread->active())
  {
    int res = read(&packet);
    if (!thread->active()) break;
    if (res == 0) continue;
    if (res < 0) break;

    if (onCapture != NULL)
    {
      packet.relay = true;
      packet.ipChanged = false;
      packet.tcpChanged = false;
      packet.udpChanged = false;
      #ifdef SNOOP_THREAD_SAFE_SUPPORT
        if (threadSafe)
          thread->synchronize(_OnCaptureCB, (void*)&packet);
        else
          onCapture(this, &packet);
      #else
        onCapture(this, &packet);
      #endif // SNOOP_THREAD_SAFE_SUPPORT
      if (packet.relay)
        relay(&packet);
    } else
    {
      relay(&packet);
    }
  }
  if (onClose != NULL) onClose(owner, this);
}

void SnoopARPSpoof::_OnCaptureCB(VDThread* thread, void* user)
{
  SnoopARPSpoof* snoopARPSpoof = (SnoopARPSpoof*)(thread->owner);
  SnoopPacket*   packet        = (SnoopPacket*)user;
  if (snoopARPSpoof->onCapture != NULL)
    snoopARPSpoof->onCapture(
      snoopARPSpoof,
      packet);
}

int SnoopARPSpoof::read(SnoopPacket* packet)
{
  int res = SnoopAdapter::read(packet);
  if (res <= 0) return res;

  if (!SnoopETH::parse(packet)) return 0;

  //
  // If ARP packet?
  //
  if (SnoopETH::isARP(packet->ethHdr, &packet->arpHdr))
  {
    preventARPRecover(packet->ethHdr, packet->arpHdr);
    return 0;
  }

  //
  // If IP packet?
  //
  if (!SnoopETH::isIP(packet->ethHdr, &packet->ipHdr)) return 0;
  packet->netType = NETTYPE_IP;

  sessionList.lock();
  SnoopARPSpoofSession* session = this->findSessionByIPPacket(packet->pktHdr, packet->ethHdr, packet->ipHdr);
  sessionList.unlock();

  if (session == NULL) return 0;

  packet->ethHdr->ether_dhost = session->targetMac;
  if (!swFilter.check(packet->pktData, packet->pktHdr->caplen))
  {
    relay(packet);
    return 0;
  }
  return res;
}

bool SnoopARPSpoof::doOpen()
{
  if (!_SnoopARPSpoof::doOpen()) return false;

  //
  // Retrieve Session List
  //
  if (!retrieveSessionList()) goto _error;

  //
  // Open swFilter
  //
  swFilter.filter = this->filter;
  if (!swFilter.open())
  {
    error = swFilter.error;
	goto _error;
  }

  //
  // Open infectThread
  //
  if (infectInterval != 0)
  {
    infectThread = new SnoopARPSpoofInfectThread(this);
    if (!infectThread->open())
    {
      SET_ERROR(SnoopError, "can not open infect thread", VDERR_UNKNOWN);
      goto _error;
    }
  }

  //
  // Send ARP infect packet
  //
  sendARPInfectAll();

  //
  // Delete recover
  //
  SAFE_DELETE(recover);

  return true;
_error:
  doClose();
  return false;
}

bool SnoopARPSpoof::doClose()
{
  //
  // Delete infectThread.
  //
  SAFE_DELETE(infectThread);

  //
  // Delete thread.
  //
  if (thread != NULL)
  {
    thread->close();
    thread->waitFor();
    delete thread;
    thread = NULL;
  }

  //
  // Send ARP recover packet
  //
  if (m_pcap != NULL)
	sendARPRecoverAll();

  //
  // Close swFilter
  //
  swFilter.close();

  //
  // Create and open recover.
  //
  SAFE_DELETE(recover);
  // ----- gilgil temp 2010.12.02 -----
  /*
  if (recoverInterval != 0)
  {
    recover = new SnoopARPRecover(this);
    recover->open();
  }
  */
  // ----------------------------------

  return _SnoopARPSpoof::doClose();
}

void SnoopARPSpoof::load(VDXML* xml)
{
  _SnoopARPSpoof::load(xml);
  infectInterval = xml->getInt("infectInterval", infectInterval);
}

void SnoopARPSpoof::save(VDXML* xml)
{
  _SnoopARPSpoof::save(xml);
  xml->setInt("infectInterval", infectInterval);
}

// ----------------------------------------------------------------------------
// SnoopARPRecover
// ----------------------------------------------------------------------------
SnoopARPRecover::SnoopARPRecover(void* owner) : _SnoopARPSpoof(owner)
{
  SnoopARPSpoof* src = (SnoopARPSpoof*)owner;

  autoCapture     = true;
  timeout         = INFINITE;
  recoverInterval = 1000; // 1 sec

  if (src == NULL) return;

  this->adapterIndex = src->adapterIndex;
#ifdef SNOOP_THREAD_SAFE_SUPPORT
  this->threadSafe = false;
#endif // SNOOP_THREAD_SAFE_SUPPORT
  this->virtualMac = src->virtualMac;

  //
  // findHost
  //
  for (int i = 0; i < src->findHost.hostList.count(); i++)
  {
    SnoopHost* host = src->findHost.hostList.at(i);
    this->findHost.hostList.add(
      host->ip,
      host->mac,
      host->name);
  }

  //
  // sessionList
  //
  for (int i = 0; i < src->sessionList.count(); i++)
  {
    SnoopARPSpoofSession* session = src->sessionList.at(i);
    this->sessionList.add(
      session->senderIP,
      session->targetIP,
      session->senderMac,
      session->targetMac);
  }
}

SnoopARPRecover::~SnoopARPRecover()
{
  close();
}

void SnoopARPRecover::run()
{
  SnoopPacket packet;

  DWORD beginTick = GetTickCount();
  DWORD lastSendARPRecoverAllTick = beginTick;

  LOG_DEBUG("[SnoopARPSpoof.cpp] SnoopARPRecover::run beg"); // gilgil temp 2009.01.28
  while (thread->active())
  {
    //
    // If time elapsed?
    //
    DWORD nowTick = GetTickCount();
    if (timeout != INFINITE && nowTick > beginTick + timeout) break;

    //
    // Send ARP recover packet periodically
    //
    if (recoverInterval != 0 && lastSendARPRecoverAllTick + recoverInterval <= nowTick)
    {
      sendARPRecoverAll();
      lastSendARPRecoverAllTick = nowTick;
    }

    //
    // Read packet
    //
    int res = SnoopAdapter::read(&packet);
    if (res == 0) continue;
    if (res < 0) break;

    //
    // If IP packet?
    //
    if (!SnoopIP::parse(&packet)) continue;
    SnoopARPSpoofSession* session = findSessionByIPPacket(packet.pktHdr, packet.ethHdr, packet.ipHdr);
    if (session != NULL)
    {
      packet.ethHdr->ether_dhost = session->targetMac;
      relay(packet.pktHdr, packet.ethHdr, packet.ipHdr);
      LOG_DEBUG("[SnoopARPSpoof.cpp] SnoopARPRecover::run relay IP packet(%s > %s)", 
        ip_to_str(session->senderIP).c_str(), ip_to_str(session->targetIP).c_str());
      sendARPRecover(session);
    }
  }
  sendARPRecoverAll();
  if (onClose != NULL) onClose(owner, this);
  LOG_DEBUG("[SnoopARPSpoof.cpp] SnoopARPRecover::run end"); // gilgil temp 2009.01.28
}

bool SnoopARPRecover::doOpen()
{
  return _SnoopARPSpoof::doOpen();
}

bool SnoopARPRecover::doClose()
{
  return _SnoopARPSpoof::doClose();
}
