// ----------------------------------------------------------------------------
//
// Snoop Component Suite version 5.0
//
// http://www.gilgil.net
//
// Copyright (c) NetworkLab Corporation All rights reserved
//
// ----------------------------------------------------------------------------

#ifndef SnoopARPSpoofH
#define SnoopARPSpoofH

#include "SnoopFindHost.h"
#include "SnoopIP.h" // for SnoopIPPacket
#include "SnoopAdapter.h"
#include "SnoopSWFilter.h"

// ----- gilgil temp 2010.11.30 -----
/*
// ----------------------------------------------------------------------------
// SnoopARPSpoofPacket
// ----------------------------------------------------------------------------
class SnoopARPSpoofPacket : public SnoopIPPacket
{
public:
  EXPORT_PROPERTY        (bool,      m_relay,      bool,      relay)
  EXPORT_POINTER_PROPERTY(bool,      m_relay,      bool,      p_relay)
  EXPORT_PROPERTY        (bool,      m_ipChanged,  bool,      ipChanged)
  EXPORT_POINTER_PROPERTY(bool,      m_ipChanged,  bool,      p_ipChanged)
  EXPORT_PROPERTY        (bool,      m_tcpChanged, bool,      tcpChanged)
  EXPORT_POINTER_PROPERTY(bool,      m_tcpChanged, bool,      p_tcpChanged)
  EXPORT_PROPERTY        (bool,      m_udpChanged, bool,      udpChanged)
  EXPORT_POINTER_PROPERTY(bool,      m_udpChanged, bool,      p_udpChanged)
};
*/
// ----------------------------------

class _SnoopARPSpoof;
class SnoopARPSpoof;

// ----------------------------------------------------------------------------
// Constant
// ----------------------------------------------------------------------------
#define SNOOP_ARP_RECOVER_DEFAULT_PORT 10063

// ----------------------------------------------------------------------------
// SnoopARPSpoofSession
// ----------------------------------------------------------------------------
class SnoopARPSpoofSession : public IVDXMLable
{
public:
  IP  senderIP;
  MAC senderMac;
  IP  targetIP;
  MAC targetMac;
public:
  SnoopARPSpoofSession();
  virtual ~SnoopARPSpoofSession();
  void clear();
public:
  virtual void load(VDXML* xml);
  virtual void save(VDXML* xml);
};

// ----------------------------------------------------------------------------
// SnoopARPSpoofSessionList
// ----------------------------------------------------------------------------
class SnoopARPSpoofSessionList : public VDLockPList<SnoopARPSpoofSession*>, public IVDXMLable
{
  friend class _SnoopARPSpoof;
protected:
  _SnoopARPSpoof* owner;
public:
  SnoopARPSpoofSessionList();
  virtual ~SnoopARPSpoofSessionList();
public:
  SnoopARPSpoofSession* add(IP senderIP,     IP targetIP,     MAC senderMac = snoopCleanMac, MAC targetMac = snoopCleanMac);
  SnoopARPSpoofSession* add(string senderIP, string targetIP, MAC senderMac = snoopCleanMac, MAC targetMac = snoopCleanMac);
  using VDLockPList<SnoopARPSpoofSession*>::del; // to remove "hides virtual function" warning? // gilgil temp 2010.12.02
  SnoopARPSpoofSession* _del(IP senderIP,     IP targetIP);
  SnoopARPSpoofSession* _del(string senderIP, string targetIP);
  SnoopARPSpoofSession* find(IP senderIP,    IP targetIP);
public:
  virtual void load(VDXML* xml);
  virtual void save(VDXML* xml);
};

// ----------------------------------------------------------------------------
// SnoopARPSpoofInfectThread
// ----------------------------------------------------------------------------
class SnoopARPSpoofInfectThread : public VDThread
{
protected:
  VDEvent *event;
public:
  SnoopARPSpoofInfectThread(SnoopARPSpoof *owner);
  virtual ~SnoopARPSpoofInfectThread();
  bool open();
  bool close();
protected:
  virtual void run();
};

// ----------------------------------------------------------------------------
// _SnoopARPSpoof
// ----------------------------------------------------------------------------
class _SnoopARPSpoof : public SnoopAdapter
{
  friend class SnoopARPSpoofSessionList; 
public:
  _SnoopARPSpoof(void* owner = NULL);
  virtual ~_SnoopARPSpoof();

protected:
  bool doOpen();
  bool doClose();

public:
  virtual int captureType() { return SCT_IN_PATH; }
  virtual int relay(SnoopPacket* packet);

public:
  int relay(PKT_HDR* pktHdr, ETH_HDR* ethHdr, IP_HDR* ipHdr);

public:
  __declspec(property (get=getAdapterIndex, put=setAdapterIndex)) int adapterIndex;
  virtual void setAdapterIndex(int value);

public:
  SnoopFindHost findHost;
  SnoopNetInfo  netInfo;
public:
  MAC  virtualMac;
  bool selfRelay;
protected:
  MAC realVirtualMac; // realVirtualMac is replaced with virtualMac in real mode.

public:
  SnoopARPSpoofSessionList sessionList;

protected:
  int sendARPInfect(int index);
  int sendARPInfect(SnoopARPSpoofSession* session);
  int sendARPInfectAll();
  int sendARPRecover(int index);
  int sendARPRecover(SnoopARPSpoofSession* session);
  int sendARPRecoverAll();
  bool retrieveSessionList();

  void preventARPRecover(ETH_HDR* ethHdr, ARP_HDR* arpHdr);
  SnoopARPSpoofSession* findSessionByIPPacket(PKT_HDR* pktHdr, ETH_HDR* ethHdr, IP_HDR* ipHdr);

public:
  virtual void load(VDXML* xml);
  virtual void save(VDXML* xml);
};

// ----------------------------------------------------------------------------
// SnoopARPSpoof
// ----------------------------------------------------------------------------
class SnoopARPRecover;
class SnoopARPSpoof : public _SnoopARPSpoof
{
  friend class SnoopARPSpoofInfectThread;
  friend class SnoopARPRecover;

public:
  SnoopARPSpoof(void* owner = NULL);
  virtual ~SnoopARPSpoof();

public:
  virtual int read(SnoopPacket* packet);

protected:
  virtual void run();
  static void _OnCaptureCB(VDThread* thread, void* user);

// ----- gilgil temp 2010.10.21 -----
/*
public:
  SnoopARPSpoof_OnCapture onCapture;
*/
// ----------------------------------

protected:
  SnoopARPRecover* recover;
  SnoopSWFilter swFilter;
  SnoopARPSpoofInfectThread* infectThread;

public:
  int infectInterval;

protected:
  virtual bool doOpen();
  virtual bool doClose();

public:
  virtual void load(VDXML* xml);
  virtual void save(VDXML* xml);
};

// ----------------------------------------------------------------------------
// SnoopARPRecover
// ----------------------------------------------------------------------------
class SnoopARPRecover : public _SnoopARPSpoof
{
public:
  SnoopARPRecover(void* owner = NULL);
  virtual ~SnoopARPRecover();
public:
  DWORD timeout;
  int   recoverInterval;
protected:
  virtual void run();
protected:
  virtual bool doOpen();
  virtual bool doClose();
};

#endif // SnoopARPSpoofH
