#include "SnoopCapture.h"

// ----------------------------------------------------------------------------
// SnoopCapture
// ----------------------------------------------------------------------------
SnoopCapture::SnoopCapture(void* owner) : VDBasicObject(owner)
{
  autoCapture  = true;
#ifdef SNOOP_THREAD_SAFE_SUPPORT
  threadSafe   = true;
#endif // SNOOP_THREAD_SAFE_SUPPORT
  thread       = NULL;
  onCapture    = NULL;
  packet.clear();
}

SnoopCapture::~SnoopCapture()
{
}

bool SnoopCapture::doOpen()
{
  if (autoCapture)
  {
    // ----- gilgil temp 2009.08.31 -----
    //
    // There can be a case that even if thread starts, state remains not vdsOpened(still vdsOpening) state.
    // So set m_state into vdsOpened before open thread.
    //
    this->m_state = vdsOpened;
    // ----------------------------------
    thread = new VDRunThread(this, this);
    thread->name = className();
    thread->open();
  }
  return true;
}

bool SnoopCapture::doClose()
{
  if (thread != NULL)
  {
    thread->waitFor();
    delete thread;
    thread = NULL;
  }
  return true;
}

int SnoopCapture::read(SnoopPacket* packet)
{
  SET_ERROR(SnoopError, "relay not supported", VDERR_NOT_READABLE);
  return -1;
}

int SnoopCapture::write(char* buf, int size)
{
  SET_ERROR(SnoopError, "relay not supported", VDERR_NOT_WRITABLE);
  return -1;
}

int SnoopCapture::relay(SnoopPacket* packet)
{
  SET_ERROR(SnoopError, "relay not supported", VDERR_NOT_SUPPORTED);
  return -1;
}

void SnoopCapture::run()
{
  // lock(); unlock(); // Wait until open() is completed. // gilgil temp 2009.08.31

  int res;

  while (thread->active())
  {
    res = read(&packet);
    if (!thread->active()) break;
    if (res == 0) continue;
    if (res < 0) break;
    
    if (onCapture != NULL)
    {
      #ifdef SNOOP_THREAD_SAFE_SUPPORT
        if (threadSafe)
          thread->synchronize(_OnCaptureCB);
       else
          onCapture(this, &packet);
      #else
        onCapture(this, &packet);
      #endif // SNOOP_THREAD_SAFE_SUPPORT
    }
  }
  if (onClose != NULL) onClose(owner, this);
}

void SnoopCapture::_OnCaptureCB(VDThread* thread, void* user)
{
  SnoopCapture* capture = (SnoopCapture*)(thread->owner);
  capture->onCapture(capture, &capture->packet);
}

void SnoopCapture::load(VDXML* xml)
{
  autoCapture = xml->getBool("autoCapture", autoCapture);
#ifdef SNOOP_THREAD_SAFE_SUPPORT
  threadSafe = xml->getBool("threadSafe", threadSafe);
#endif // SNOOP_THREAD_SAFE_SUPPORT
}

void SnoopCapture::save(VDXML* xml)
{
  xml->setBool("autoCapture", autoCapture);
#ifdef SNOOP_THREAD_SAFE_SUPPORT
  xml->setBool("threadSafe", threadSafe);
#endif // SNOOP_THREAD_SAFE_SUPPORT
}