#include "SnoopDump.h"

// ----------------------------------------------------------------------------
// SnoopDump
// ----------------------------------------------------------------------------
SnoopDump::SnoopDump(void* owner)
{
  this->owner = owner;
  m_active = false;
  m_error = "";
  m_pcap = NULL;
}

SnoopDump::~SnoopDump()
{
  close();
  if (m_pcap != NULL)
  {
    pcap_close(m_pcap);
    m_pcap = NULL;
  }
}

bool SnoopDump::open(string fileName)
{
  if (m_pcap == NULL) m_pcap = pcap_open_dead(DLT_EN10MB, SNOOP_DEFAULT_SNAPLEN);  
  return open(m_pcap, fileName);
}

bool SnoopDump::open(pcap* pcap, string fileName)
{
  if (m_active) goto _error;
  if (fileName == "")
  {
    m_error = "snoop error: invalid file name";
    goto _error;
  }
  pcap_dumper = pcap_dump_open(pcap, fileName.c_str());
  if (pcap_dumper == NULL)
  {
    m_error = pcap_geterr(pcap);
    LOG_ERROR("[SnoopDump.cpp] SnoopDump::open pcap_dump_open error=%s", m_error.c_str());
    goto _error;
  }

  m_active = true;
  return true;

_error:
  return false;
}

bool SnoopDump::close()
{
  if (!m_active) goto _error;
  pcap_dump_close(pcap_dumper);
  m_active = false;
  return true;
_error:
  return false;
}

void SnoopDump::write(PKT_HDR* pktHdr, char* data)
{
  pcap_dump((u_char*)pcap_dumper, pktHdr, (const u_char*)data);
}