#include "SnoopFile.h"
#include "VDFile.h" // for VDERR_FILENAME_NOT_SPECIFIED

REGISTER_METACLASS(SnoopFile, SnoopCapture)

// ----------------------------------------------------------------------------
// SnoopFile
// ----------------------------------------------------------------------------
SnoopFile::SnoopFile(void* owner) : SnoopPcap(owner)
{
  fileName  = "";
  timeSync  = false;
  startTS   = 0;
  startTick = 0;
}

SnoopFile::~SnoopFile()
{
  close();
}

bool SnoopFile::doOpen()
{
  if (fileName == "")
  {
    SET_ERROR(VDFileError, "file name not specified", VDERR_FILENAME_NOT_SPECIFIED);
    return false;
  }
  if (!pcapOpen((char*)("file://" + fileName).c_str(), NULL, NULL))
  {
    return false;
  }

  if (timeSync)
  {
    mmObject.open();
    startTS = 0;
    startTick = mmObject.tick();
  }

  return SnoopPcap::doOpen();
}

bool SnoopFile::doClose()
{
  if (timeSync)
  {
    mmObject.close();
  }
  return SnoopPcap::doClose();
}

int SnoopFile::read(SnoopPacket* packet)
{
  int res = SnoopPcap::read(packet);
  if (timeSync)
  {
    DWORD nowTS = packet->pktHdr->ts.tv_sec * 1000 + packet->pktHdr->ts.tv_usec / 1000;
    if (startTS == 0) startTS = nowTS;
    DWORD elapsedTS = nowTS - startTS;
    while (true)
    {
      DWORD nowTick = mmObject.tick();
      DWORD elapsedTick = nowTick - startTick;
      if (elapsedTick >= elapsedTS) break;
      mmObject.sleep(1);
    }
  }

  return res;
}

void SnoopFile::load(VDXML* xml)
{
  SnoopPcap::load(xml);
  fileName = xml->getStr("fileName", fileName);
  timeSync = xml->getBool("timeSync", timeSync);
}

void SnoopFile::save(VDXML* xml)
{
  SnoopPcap::save(xml);
  xml->setStr("fileName", fileName);
  xml->setBool("timeSync", timeSync);
}