#include "SnoopFindHost.h"

// ----------------------------------------------------------------------------
// SnoopFindHost
// ----------------------------------------------------------------------------
#pragma pack(push, 1)
typedef struct
{
  IP  ip;
  MAC mac;
} SnoopFineHostPacketParam;
#pragma pack(pop)

void SnoopFindHost::run()
{
  // lock(); unlock(); // Wait until open() is completed. // gilgil temp 2009.08.31

  int res;
  SnoopFineHostPacketParam param;

  while (thread->active())
  {
    res = readIPAndMAC(&param.ip, &param.mac);
    if (!thread->active()) break;
    if (res == 0) continue;
    if (res < 0) break;

    if (onCapture != NULL)
    {
      #ifdef SNOOP_THREAD_SAFE_SUPPORT
        if (threadSafe)
          thread->synchronize(_OnCaptureCB, (void*)&param);
       else
          onCapture(this, &param.ip, &param.mac);
      #else
        onCapture(this, &param.ip, &param.mac);
      #endif // SNOOP_THREAD_SAFE_SUPPORT
    }
  }
}

void SnoopFindHost::_OnCaptureCB(VDThread * thread, void* user)
{
  SnoopFindHost* snoopFindHost = (SnoopFindHost*)(thread->owner);
  SnoopFineHostPacketParam* param = (SnoopFineHostPacketParam*)user;
  if (snoopFindHost->onCapture != NULL)
    snoopFindHost->onCapture(
      snoopFindHost,
      &param->ip,
      &param->mac);
}

int SnoopFindHost::readIPAndMAC(IP* ip, MAC* mac)
{
  SnoopPacket packet;

  int res = SnoopAdapter::read(&packet);
  if (scanInterval != 0)
  {
    DWORD nowTick = GetTickCount();
    if (nowTick - lastSendTick > (DWORD)scanInterval)
    {
      sendARPRequestAll();
    }
  }
  if (res <= 0) return res;
  
  if (!SnoopARP::parse(&packet)) return 0;
  bool ok = false;
  if (ntohs(packet.arpHdr->ar_op) == ARPOP_REPLY && (MAC)packet.ethHdr->ether_dhost == netInfo.mac)
  {
    *ip  = htonl(packet.arpHdr->ar_si);
    *mac = packet.arpHdr->ar_sa;
    for (int i = 0; i < hostList.count(); i++)
    {
      SnoopHost* host = hostList.at(i);
      if (host->ip == *ip)
      {
        host->mac = *mac;
        ok = true;
      }
    }
  }
  return ok ? res : 0;
}

SnoopFindHost::SnoopFindHost(void* owner) : SnoopAdapter(owner)
{
  filter = "arp";
  onCapture = NULL;
  lastSendTick = 0;
  scanInterval = 0;
  sendInterval = 0;
}

SnoopFindHost::~SnoopFindHost()
{
  close();
  // ----- gilgil temp 2008.11.23 -----
  /*
  SAFE_DELETE(netInfo);
  SAFE_DELETE(hostList);
  */
  // ----------------------------------
}

int SnoopFindHost::sendARPRequest(IP ip)
{
  ETH_HDR ethHdr;
  ARP_HDR arpHdr;

  //
  // Set Ethernet Hdr
  //
  ethHdr.ether_dhost = snoopBroadcastMac;
  ethHdr.ether_shost = netInfo.mac;
  ethHdr.ether_type  = htons(ETHERTYPE_ARP);

  //
  // Set ARP Hdr
  //
  arpHdr.ar_hrd = htons(ARPHRD_ETHER);
  arpHdr.ar_pro = htons(2048);
  arpHdr.ar_hln = 6;
  arpHdr.ar_pln = 4;
  arpHdr.ar_op = htons(ARPOP_REQUEST);
  arpHdr.ar_sa = netInfo.mac;
  arpHdr.ar_si = htonl(netInfo.ip);
  arpHdr.ar_ta = snoopCleanMac;
  arpHdr.ar_ti = htonl(ip);

  //
  // Send Raw Packet
  //
  return snoopSendARP(m_pcap, &ethHdr, &arpHdr);
}

void SnoopFindHost::setAdapterIndex(int value)
{
  SnoopAdapter::setAdapterIndex(value);
  netInfo.setAdapterIndex(value);
}

bool SnoopFindHost::doOpen()
{
  lastSendTick = GetTickCount();
  return SnoopAdapter::doOpen();
}

bool SnoopFindHost::doClose()
{
  lastSendTick = 0;
  return SnoopAdapter::doClose();
}

bool SnoopFindHost::sendARPRequestAll()
{
  for (int i = 0; i < hostList.count(); i++)
  {
    SnoopHost* host = hostList.at(i);
    if (host->mac.isClear())
    {
      if (sendARPRequest(host->ip) < 0) return false;
      if (sendInterval != 0)
        Sleep(sendInterval);
    }
  }
  lastSendTick = GetTickCount();
  return true;
}

bool SnoopFindHost::isFoundAll()
{
  for (int i = 0; i < hostList.count(); i++)
  {
    SnoopHost* host = hostList.at(i);
    if (host->mac.isClear())
      return false;
  }
  return true;
}

void SnoopFindHost::logUnfoundHost(VDLog* log)
{
  if (log == NULL) log = VDLog::getLog();
  if (log == NULL) return;
  for (int i = 0; i < hostList.count(); i++)
  {
    SnoopHost* host = hostList.at(i);
    if (host->mac.isClear())
    {
      log->info("[SnoopFindHost.cpp] SnoopFindHost::logUnfoundHost ip=%s", ip_to_str(host->ip).c_str());
    }
  }
}

bool SnoopFindHost::scan(DWORD timeout)
{
  if (isFoundAll()) return true;

  if (!sendARPRequestAll()) return false;

  //
  // Receive and wait
  //
  DWORD startTick = GetTickCount();
  DWORD nowTick;
  while (true)
  {
    //
    // If found all?
    //
    if (isFoundAll()) return true;

    //
    // Is time elapsed?
    //
    nowTick = GetTickCount();
    if (nowTick - startTick > timeout)
    {
      SET_ERROR(SnoopError, "can not find all host", VDERR_UNKNOWN);
      logUnfoundHost();
      return false;
    }

    //
    // Retrieve ARP packet
    //
    IP  ip;
    MAC mac;
    int res = readIPAndMAC(&ip, &mac);
    if (res == 0) continue;
    if (res < 0) break;
  }
  return false;
}

void SnoopFindHost::load(VDXML* xml)
{
  SnoopAdapter::load(xml);
  scanInterval = xml->getInt("scanInterval", scanInterval);
  sendInterval = xml->getInt("sendInterval", sendInterval);
  // hostList.load(xml->gotoChild("HostList")); // gilgil temp 2009.01.16
}

void SnoopFindHost::save(VDXML* xml)
{
  SnoopAdapter::save(xml);
  xml->setInt("scanInterval", scanInterval);
  xml->setInt("sendInterval", sendInterval);
  // hostList.save(xml->gotoChild("HostList")); // gilgil temp 2009.01.16
}
