#include "SnoopHostList.h"

// ----------------------------------------------------------------------------
// SnoopHost
// ----------------------------------------------------------------------------
SnoopHost::SnoopHost()
{
  clear();
}

SnoopHost::~SnoopHost()
{
  clear();
}

void SnoopHost::clear()
{
  ip   = (UINT32)0;
  mac  = snoopCleanMac;
  name = "";
}

void SnoopHost::load(VDXML* xml)
{
  ip   = str_to_ip(xml->getStr("ip", ip_to_str(ip)));
  mac  = (string)xml->getStr("mac", "");
  name = xml->getStr("name", "");
}

void SnoopHost::save(VDXML* xml)
{
  if (ip != 0) xml->setStr("ip", ip_to_str(ip));
  if (!mac.isClear())  xml->setStr("mac", mac);
  if (name != "")      xml->setStr("name", name);
}

// ----------------------------------------------------------------------------
// SnoopHostList
// ----------------------------------------------------------------------------
SnoopHostList::SnoopHostList()
{
  clear();
}

SnoopHostList::~SnoopHostList()
{
  clear();
}

SnoopHost* SnoopHostList::add(IP ip, MAC mac, string name)
{
  if (findByIP(ip)) return NULL; // if already exists
  SnoopHost* host = new SnoopHost;
  host->ip = ip;
  host->mac = mac;
  host->name = name;
  if (VDPList<SnoopHost*>::add(host) == -1)
  {
    delete host;
    return NULL;
  }
  return host;
}

SnoopHost* SnoopHostList::add(string ip)
{
  return add(str_to_ip(ip));
}

SnoopHost* SnoopHostList::findByIP(IP ip)
{
  SnoopHost* res;

  for (int i = 0; i < count(); i++)
  {
    res = at(i);
    if (res->ip == ip) return res;
  }
  return NULL;
}

void SnoopHostList::load(VDXML* xml)
{
  clear();
  for (int i = 0; i < xml->getChildCount(); i++)
  {
    VDXML* childXML = xml->getChild(i);
    SnoopHost* host = add();
    host->load(childXML);
  }
}

void SnoopHostList::save(VDXML* xml)
{
  xml->clearChild();
  for (int i = 0; i < count(); i++)
  {
    SnoopHost* host = at(i);
    if (host->mac.isClear()) continue;
    VDXML* childXML = xml->addChild("Host");
    host->save(childXML);
  }
}
