#include "SnoopIP.h"

// ----------------------------------------------------------------------------
// SnoopIP
// ----------------------------------------------------------------------------
bool SnoopIP::is(IP_HDR* ipHdr, UINT8 protocol, void** transportHdr)
{
  if (ipHdr->ip_p != protocol)
    return false;
  if (transportHdr != NULL)
    *transportHdr = (void*)((char*)(ipHdr) + ipHdr->ip_hl * sizeof(UINT32));
  return true;
}

bool SnoopIP::isTCP(IP_HDR* ipHdr, TCP_HDR** tcpHdr)
{
  if (ipHdr->ip_p != IPPROTO_TCP)
    return false;
  if (tcpHdr != NULL)
    *tcpHdr = (TCP_HDR*)((char*)(ipHdr) + ipHdr->ip_hl * sizeof(UINT32));
  return true;
}

bool SnoopIP::isUDP(IP_HDR* ipHdr, UDP_HDR** udpHdr)
{
  if (ipHdr->ip_p != IPPROTO_UDP)
    return false;
  if (udpHdr != NULL)
    *udpHdr = (UDP_HDR*)((char*)(ipHdr) + ipHdr->ip_hl * sizeof(UINT32));
  return true;
}

bool SnoopIP::isICMP(IP_HDR* ipHdr, ICMP_HDR** icmpHdr)
{
  if (ipHdr->ip_p != IPPROTO_ICMP)
    return false;
  if (icmpHdr != NULL)
    *icmpHdr = (ICMP_HDR*)((char*)(ipHdr) + ipHdr->ip_hl * sizeof(UINT32));
  return true;
}

//
// All ipHdr field except ipHdr.ip_sum
//
UINT16 SnoopIP::checksum(IP_HDR* ipHdr)
{
  int i;
  UINT32 sum;
  UINT16 *p;

  sum = 0;

  // Add ipHdr buffer as array of UINT16
  p = (UINT16*)(ipHdr);
  for (i = 0; i < (int)sizeof(IP_HDR) / 2; i++)
  {
    sum += ntohs(*p);
    p++;
  }
  
  // Do not consider padding because ip header length is always multilpe of 2.

  // Decrease checksum from sum
  sum -= ntohs(ipHdr->ip_sum);

  // Recalculate sum
  while(sum >> 16)
  {
    sum = (sum & 0xFFFF) + (sum >> 16);
  }
  sum = ~sum;

  return (UINT16)sum;
}

//
// Calculate Checksum on condition that only one value(UINT16) is changed.
//
UINT16 SnoopIP::recalculateChecksum(UINT16 oldChecksum, UINT16 oldValue, UINT16 newValue)
{
  UINT32 sum;

  sum = oldValue + (~newValue & 0xFFFF);
  sum += oldChecksum;
  sum = (sum & 0xFFFF) + (sum >> 16);
  return (UINT16)(sum + (sum >> 16));
}

bool SnoopIP::parse(SnoopPacket* packet)
{
  if (!SnoopETH::parse(packet)) return false;

  if (!SnoopETH::isIP(packet->ethHdr, &packet->ipHdr)) return false;
  packet->netType = NETTYPE_IP;

  return true;
}
