#include "SnoopNetInfo.h"

// ----------------------------------------------------------------------------
// SnoopNetInfo
// ----------------------------------------------------------------------------
SnoopNetInfo::SnoopNetInfo(void* owner)
{
  this->owner  = owner;
  m_adapterIndex = SNOOP_INVALID_ADAPTER_INDEX;
  clear();
}

SnoopNetInfo::~SnoopNetInfo()
{
}

void SnoopNetInfo::clear()
{
  ip = (UINT32)0;
  mac = snoopCleanMac;
  subnetMask = (UINT32)0;
  gateway = (UINT32)0;
  startIP = (UINT32)0;
  endIP = (UINT32)0;
  ip_and_subnetMask = (UINT32)0;
}

int SnoopNetInfo::getAdapterIndex()
{
  return m_adapterIndex;
}

void SnoopNetInfo::setAdapterIndex(int value)
{
  if (m_adapterIndex == value) return;
  clear();
  m_adapterIndex = value;
  SnoopAdapterInfo* snoopAdapterInfo = snoopGetAdapterInfo(m_adapterIndex);
  if (snoopAdapterInfo == NULL)
  {
    LOG_ERROR("[SnoopNetInfo.cpp] SnoopNetInfo::setAdapterIndex snoopGetAdapterInfo(%d) return NULL", m_adapterIndex);
    return;
  }

  PIP_ADAPTER_INFO p = snoopAdapterInfo->ipAdapterInfo;
  if (p == NULL)
  {
    LOG_ERROR("[SnoopNetInfo.cpp] SnoopNetInfo::setAdapterIndex snoopAdapterInfo->adapterInfo is NULL (index=%d)", m_adapterIndex);
    return;
  }

  // ip
  if (p->IpAddressList.IpAddress.String[0] != '\0')
    ip = str_to_ip(p->IpAddressList.IpAddress.String);
  // mac
  for (int i = 0; i < SNOOP_MAC_SIZE; i++)
    mac.s[i] = p->Address[i];

  // subnetMask
  if (p->IpAddressList.IpMask.String[0] != '\0')
    subnetMask = str_to_ip(p->IpAddressList.IpMask.String);

  // gateway
  if (p->GatewayList.IpAddress.String[0] != '\0')
    gateway = str_to_ip(p->GatewayList.IpAddress.String);

  // startIP
  startIP = (ip & subnetMask) + 1;

  // endIP
  endIP = (ip | ~subnetMask);

  // ip_and_subnetMask
  ip_and_subnetMask = ip & subnetMask;
}

void SnoopNetInfo::load(VDXML* xml)
{
  //
  // netInfo
  //
  ip         = str_to_ip(xml->getStr("ip", ip_to_str(ip)));
  mac        = xml->getStr("mac", mac);
  subnetMask = str_to_ip(xml->getStr("subnetMask", ip_to_str(subnetMask)));
  gateway    = str_to_ip(xml->getStr("gateway", ip_to_str(gateway)));
  startIP    = str_to_ip(xml->getStr("startIP", ip_to_str(startIP)));
  endIP      = str_to_ip(xml->getStr("endIP", ip_to_str(endIP)));
  ip_and_subnetMask = ip & subnetMask;
}

void SnoopNetInfo::save(VDXML* xml)
{
  //
  // netInfo
  //
  xml->setStr("ip", ip_to_str(ip));
  xml->setStr("mac", mac);
  xml->setStr("subnetMask", ip_to_str(subnetMask));
  xml->setStr("gateway", ip_to_str(gateway));
  xml->setStr("startIP", ip_to_str(startIP));
  xml->setStr("endIP", ip_to_str(endIP));
}
