// ----------------------------------------------------------------------------
//
// Snoop Component Suite version 5.0
//
// http://www.gilgil.net
//
// Copyright (c) NetworkLab Corporation All rights reserved
//
// ----------------------------------------------------------------------------

#ifndef SnoopPacketH
#define SnoopPacketH

#include "SnoopType.h"

// ----------------------------------------------------------------------------
// SnoopLinkType
// ----------------------------------------------------------------------------
typedef enum
{
  LINKTYPE_UNKNOWN = -1,
  LINKTYPE_ETH,
  LINKTYPE_FDDI
} SnoopLinkType;

// ----------------------------------------------------------------------------
// SnoopNetType
// ----------------------------------------------------------------------------
typedef enum
{
  NETTYPE_UNKNOWN = -1,
  NETTYPE_IP,
  NETTYPE_ARP
} SnoopNetType;

// ----------------------------------------------------------------------------
// SnoopNetType
// ----------------------------------------------------------------------------
typedef enum
{
  PROTOTYPE_UNKNOWN = -1,
  PROTOTYPE_TCP,
  PROTOTYPE_UDP,
  PROTOTYPE_ICMP
} SnoopProtoType;

// ----------------------------------------------------------------------------
// SnoopPacket
// ----------------------------------------------------------------------------
#pragma pack(push, 1)
class SnoopPacket
{
public:
  ///
  /// packet
  ///
  PKT_HDR*       pktHdr;
  char*          pktData;

  ///
  /// datalink layer
  ///
  SnoopLinkType  linkType;
  ETH_HDR*       ethHdr;
  FDDI_HDR*      fddiHdr;

  ///
  /// network layer
  ///
  SnoopNetType   netType;
  IP_HDR*        ipHdr;
  ARP_HDR*       arpHdr;


  ///
  /// transport layer(protocol)
  ///
  SnoopProtoType protoType;
  TCP_HDR*       tcpHdr;
  UDP_HDR*       udpHdr;
  ICMP_HDR*      icmpHdr;

  ///
  /// tcp option
  ///
  void*          tcpOption;

  ///
  /// payload
  ///
  char*          payload;
  int            payloadLen;

  ///
  /// control
  ///
  bool           relay;
  bool           ipChanged;
  bool           tcpChanged;
  bool           udpChanged;

public:
  void           clear();
};
#pragma pack (pop)

// ----- gilgil temp 2010.11.30 -----
/*
class SnoopAllPacket : public SnoopPacket
{
public:
  EXPORT_PROPERTY        (ETH_HDR*,  m_ethHdr,     ETH_HDR*,  ethHdr)
  EXPORT_POINTER_PROPERTY(ETH_HDR*,  m_ethHdr,     ETH_HDR*,  p_ethHdr)
  EXPORT_PROPERTY        (FDDI_HDR*, m_fddiHdr,    FDDI_HDR*, fddiHdr)
  EXPORT_POINTER_PROPERTY(FDDI_HDR*, m_fddiHdr,    FDDI_HDR*, p_fddiHdr)

  EXPORT_PROPERTY        (IP_HDR*,   m_ipHdr,      IP_HDR*,   ipHdr)
  EXPORT_POINTER_PROPERTY(IP_HDR*,   m_ipHdr,      IP_HDR*,   p_ipHdr)
  EXPORT_PROPERTY        (ARP_HDR*,  m_arpHdr,     ARP_HDR*,  arpHdr)
  EXPORT_POINTER_PROPERTY(ARP_HDR*,  m_arpHdr,     ARP_HDR*,  p_arpHdr)

  EXPORT_PROPERTY        (TCP_HDR*,  m_tcpHdr,     TCP_HDR*,  tcpHdr)
  EXPORT_POINTER_PROPERTY(TCP_HDR*,  m_tcpHdr,     TCP_HDR*,  p_tcpHdr)
  EXPORT_PROPERTY        (UDP_HDR*,  m_udpHdr,     UDP_HDR*,  udpHdr)
  EXPORT_POINTER_PROPERTY(UDP_HDR*,  m_udpHdr,     UDP_HDR*,  p_udpHdr)
  EXPORT_PROPERTY        (ICMP_HDR*, m_icmpHdr,    ICMP_HDR*, icmpHdr)
  EXPORT_POINTER_PROPERTY(ICMP_HDR*, m_icmpHdr,    ICMP_HDR*, p_icmpHdr)

  EXPORT_PROPERTY        (void*,     m_tcpOption,  void*,     tcpOption)
  EXPORT_POINTER_PROPERTY(void*,     m_tcpOption,  void*,     p_tcpOption)
  EXPORT_PROPERTY        (char*,     m_tcpData,    char*,     tcpData)
  EXPORT_POINTER_PROPERTY(char*,     m_tcpData,    char*,     p_tcpData)
  EXPORT_PROPERTY        (int,       m_tcpDataLen, int,       tcpDataL)
  EXPORT_POINTER_PROPERTY(int,       m_tcpDataLen, int,       p_tcpDataL)
  EXPORT_PROPERTY        (char*,     m_udpData,    char*,     udpData)
  EXPORT_POINTER_PROPERTY(char*,     m_udpData,    char*,     p_udpData)
  EXPORT_PROPERTY        (int,       m_udpDataLen, int,       udpDataL)
  EXPORT_POINTER_PROPERTY(int,       m_udpDataLen, int,       p_udpDataL)

  EXPORT_PROPERTY        (bool,      m_relay,      bool,      relay)
  EXPORT_POINTER_PROPERTY(bool,      m_relay,      bool,      p_relay)
  EXPORT_PROPERTY        (bool,      m_ipChanged,  bool,      ipChanged)
  EXPORT_POINTER_PROPERTY(bool,      m_ipChanged,  bool,      p_ipChanged)
  EXPORT_PROPERTY        (bool,      m_tcpChanged, bool,      tcpChanged)
  EXPORT_POINTER_PROPERTY(bool,      m_tcpChanged, bool,      p_tcpChanged)
  EXPORT_PROPERTY        (bool,      m_udpChanged, bool,      udpChanged)
  EXPORT_POINTER_PROPERTY(bool,      m_udpChanged, bool,      p_udpChanged)
};
*/
// ----------------------------------

#endif  SnoopPacketH
