#include "SnoopPcap.h"

// ----------------------------------------------------------------------------
// SnoopPcap
// ----------------------------------------------------------------------------
SnoopPcap::SnoopPcap(void* owner) : SnoopCapture(owner)
{
  filter       = "";
  snapLen      = SNOOP_DEFAULT_SNAPLEN;
  flags        = PCAP_OPENFLAG_PROMISCUOUS;
  readTimeout  = SNOOP_DEFAULT_READTIMEOUT;
  m_pcap       = NULL;
  m_dataLink   = DLT_NULL;
  m_sourceName = "";
}

SnoopPcap::~SnoopPcap()
{
  close();
}

bool SnoopPcap::doOpen()
{
  return SnoopCapture::doOpen();
}

bool SnoopPcap::doClose()
{
  // ----- by gilgil 2009.09.01 -----
  //
  // Strange to say, when pcap_next_ex is called after pcap_close is called, it occurs memory problem.
  // So waits until thread is terminated.
  //
  if (thread != NULL)
  {
    thread->close(false);
    thread->waitFor();
  }
  // --------------------------------
  if (m_pcap != NULL)
  {
    pcap_close(m_pcap);
    m_pcap       = NULL;
  }
  m_dataLink   = DLT_NULL;
  m_sourceName = "";

  return SnoopCapture::doClose();
}

int SnoopPcap::read(SnoopPacket* packet)
{
  if (m_state != vdsOpened)
  {
    SET_ERROR(VDError, format("not opened state(%s)", className()), VDERR_NOT_OPENED_STATE);
    return -1;
  }
  int i = pcap_next_ex(m_pcap, (pcap_pkthdr**)&packet->pktHdr, (const u_char**)&packet->pktData);
  switch (i) {
    case -2: // if EOF was reached reading from an offline capture
      SET_ERROR(SnoopError, format("pcap_next_ex return -2(%s)", pcap_geterr(m_pcap)), VDERR_UNKNOWN);
      break;
    case -1: // if an error occurred
      SET_ERROR(SnoopError, format("pcap_next_ex return -1(%s)", pcap_geterr(m_pcap)), VDERR_UNKNOWN);
      break;
    case 0 : // if the timeout occurs
      break;
  }
  return i;
}

// returns 0 on success and -1 on failure.
int SnoopPcap::write(char* buf, int size)
{
  return pcap_sendpacket(m_pcap, (const u_char*)buf, size);
}

int SnoopPcap::relay(SnoopPacket* packet)
{
  SET_ERROR(SnoopError, "relay not supported", VDERR_NOT_SUPPORTED);
  return -1;
}

// ----- gilgil temp 2009.08.30 -----
/*
int SnoopPcap::write(char* buf, int size)
{
  pcap_send_queue *squeue;
  int res;
  struct pcap_pkthdr pktHdr;

  squeue = pcap_sendqueue_alloc(size + 1000);

  pktHdr.ts.tv_sec = 0;
  pktHdr.ts.tv_usec = 0;
  pktHdr.caplen = size;
  res = pcap_sendqueue_queue(squeue, &pktHdr, (const u_char *)buf);
  if (res == -1) goto _end;

  res = pcap_sendqueue_transmit(pcap, squeue, TRUE);
_end:
  pcap_sendqueue_destroy(squeue);
  return res;
}
*/
// ----------------------------------

bool SnoopPcap::pcapOpen(char* source, pcap_rmtauth* auth, pcap_if_t* dev)
{
  char errBuf[PCAP_ERRBUF_SIZE];

  m_pcap = pcap_open(source, snapLen, flags, readTimeout, auth, errBuf);
  if (m_pcap == NULL)
  {
    SET_ERROR(SnoopError, format("error in pcap_open(%s)", errBuf), VDERR_UNKNOWN);
    return false;
  }
  m_dataLink = pcap_datalink(m_pcap);
  if (m_dataLink != DLT_EN10MB)
  {
    LOG_INFO("[SnoopCapture.cpp] SnoopCapture::doOpen() pcap_datalink return =%d(0x%x)", m_dataLink, m_dataLink);
  }
  m_sourceName = (string)source;
  if (!pcapProcessFilter(dev)) return false;

  return true;
}

bool SnoopPcap::pcapProcessFilter(pcap_if_t* dev)
{
  u_int uNetMask;
  bpf_program code;

  if ((dev != NULL) && (dev->addresses != NULL))
    uNetMask = ((struct sockaddr_in*)(dev->addresses->netmask))->sin_addr.S_un.S_addr;
  else
    uNetMask = 0xFFFFFFFF;
  if (pcap_compile(m_pcap, &code, (char*)filter.c_str(), 1, uNetMask) < 0 )
  {
    SET_ERROR(SnoopError, format("error in pcap_compile(%s)", pcap_geterr(m_pcap)), VDERR_UNKNOWN);
    return false;
  }
  if (pcap_setfilter(m_pcap, &code) < 0)
  {
    SET_ERROR(SnoopError, format("error in pcap_setfilter(%s)", pcap_geterr(m_pcap)), VDERR_UNKNOWN);
    return false;
  }
  return true;
}

void SnoopPcap::load(VDXML* xml)
{
  SnoopCapture::load(xml);
  filter = xml->getStr("filter", filter);
  snapLen = xml->getInt("snapLen", snapLen);
  flags = xml->getInt("flags", flags);
  readTimeout = xml->getInt("readTimeout", readTimeout);
}

void SnoopPcap::save(VDXML* xml)
{
  SnoopCapture::save(xml);
  xml->setStr("filter", filter);
  xml->setInt("snapLen", snapLen);
  xml->setInt("flags", flags);
  xml->setInt("readTimeout", readTimeout);
}