#include "SnoopRemote.h"

REGISTER_METACLASS(SnoopRemote, SnoopCapture)

// ----------------------------------------------------------------------------
// SnoopRemote
// ----------------------------------------------------------------------------
SnoopRemote::SnoopRemote(void* owner) : SnoopPcap(owner)
{
  readTimeout = 1000;
  host = "";
  userName = "";
  password = "";
  adapterIndex = SNOOP_INVALID_ADAPTER_INDEX;
  adaptersInfo = new SnoopAdaptersInfo;
  onGetAdapterInfo = NULL;
}

SnoopRemote::~SnoopRemote()
{
  close();
  SAFE_DELETE(adaptersInfo);
}

bool SnoopRemote::doOpen()
{
  int i;
  string source;
  pcap_rmtauth auth;
  pcap_if_t *dev;
  char errBuf[PCAP_ERRBUF_SIZE];
  SnoopAdapterInfo* adapterInfo;

  source = "rpcap://" + host + "/";
  auth.type = RPCAP_RMTAUTH_PWD;
  auth.username = (char*)userName.c_str();
  auth.password = (char*)password.c_str();

  i = pcap_findalldevs_ex((char*)source.c_str(), &auth, &adaptersInfo->allDevs, errBuf);
  if (i != 0) // if error occured
  {
    SET_ERROR(SnoopError, format("error in pcap_findalldevs_ex(%s)", errBuf), VDERR_UNKNOWN);
    goto _error;
  }

  dev = adaptersInfo->allDevs;
  i = 0;
  while (dev != NULL)
  {
    SnoopAdapterInfo* adapterInfo = new SnoopAdapterInfo;

    adapterInfo->index = i;
    adapterInfo->name = dev->name;
    adapterInfo->description = dev->description;
    adapterInfo->dev = dev;

    adaptersInfo->items.add(adapterInfo);

    dev = dev->next;
    i++;
  }

  if (onGetAdapterInfo != NULL)
  {
    onGetAdapterInfo(this, adaptersInfo, &adapterIndex);
  }
  if (adapterIndex == SNOOP_INVALID_ADAPTER_INDEX)
  {
    SET_ERROR(SnoopError, "cancel by user or invalid adapter index", VDERR_UNKNOWN);
    goto _error;
  }

  adapterInfo = adaptersInfo->items.at(adapterIndex);
  if (adapterInfo == NULL)
  {
    SET_ERROR(SnoopError, "cancel by user or invalid adapter index", VDERR_UNKNOWN);
    goto _error;
  }
  source = adapterInfo->name;
  if (!pcapOpen((char*)source.c_str(), &auth, adapterInfo->dev))
  {
    goto _error;
  }

  return SnoopPcap::doOpen();
_error:
  return false;
}

bool SnoopRemote::doClose()
{
  return SnoopPcap::doClose();
}

void SnoopRemote::load(VDXML* xml)
{
  SnoopPcap::load(xml);
  host = xml->getStr("host", host);
  userName = xml->getStr("userName", userName);
  password = xml->getStr("password", password);
  adapterIndex = xml->getInt("adapterIndex", adapterIndex);
}

void SnoopRemote::save(VDXML* xml)
{
  SnoopPcap::save(xml);
  xml->setStr("host", host);
  xml->setStr("userName", userName);
  xml->setStr("password", password);
  xml->setInt("adapterIndex", adapterIndex);
}