#include "SnoopSWFilter.h"

// ----------------------------------------------------------------------------
// SnoopSWFilter
// ----------------------------------------------------------------------------
SnoopSWFilter::SnoopSWFilter(void* owner) : VDBasicObject(owner)
{
  m_pcap = NULL;
  m_code = NULL;
  filter = "";
}

SnoopSWFilter::~SnoopSWFilter()
{
  close();
}

bool SnoopSWFilter::doOpen()
{
  int res;

  m_pcap = pcap_open_dead(DLT_EN10MB, SNOOP_DEFAULT_SNAPLEN); // gilgil temp 2010.09.24
  if (m_pcap == NULL)
  {
    SET_ERROR(SnoopError, "error in pcap_open_dead return NULL", VDERR_UNKNOWN);
    goto _error;
  }

  m_code = (bpf_program*)malloc(sizeof(bpf_program));
  res = pcap_compile(m_pcap, m_code, filter.c_str(), 1, 0xFFFFFFFF);
  if (res < 0)
  {
    SET_ERROR(SnoopError, format("error in pcap_compile(%s)", pcap_geterr(m_pcap)), VDERR_UNKNOWN);
    goto _error;
 }

  return true;
_error:
  doClose();
  return false;
}

bool SnoopSWFilter::doClose()
{
  if (m_pcap != NULL)
  {
    pcap_close(m_pcap);
    m_pcap = NULL;
  }
  if (m_code != NULL)
  {
    pcap_freecode(m_code);
    free(m_code);		
    m_code = NULL;
  }
  return true;
}

bool SnoopSWFilter::check(char* pktData, int pktLen)
{
  if (m_state != vdsOpened)
  {
    SET_ERROR(VDError, format("not opened state(%s %s)", className(), name.c_str()), VDERR_NOT_OPENED_STATE);
    return false;
  }
  LOG_ASSERT(m_pcap != NULL && m_code != NULL);
  int res = bpf_filter(m_code->bf_insns, (const u_char*)pktData, pktLen, pktLen);
  return res > 0;
}
