#include "SnoopTCP.h"

// ----------------------------------------------------------------------------
// SnoopTCP
// ----------------------------------------------------------------------------
bool SnoopTCP::isData(IP_HDR* ipHdr, TCP_HDR* tcpHdr, char** tcpData, int* tcpDataLen)
{
  int   tcpHdrLen   = tcpHdr->th_off * sizeof(UINT32);
  char* _tcpData    = (char*)(tcpHdr) + tcpHdrLen;
  int   _tcpDataLen = ntohs(ipHdr->ip_len) - sizeof(TCP_HDR) - tcpHdrLen;
  
  if (_tcpDataLen > 0)
  {
    if (tcpData != NULL)
      *tcpData = _tcpData;
    if (tcpDataLen != NULL)
      *tcpDataLen = _tcpDataLen;
    return true;
  }
  return false;
}

bool SnoopTCP::isOption(TCP_HDR* tcpHdr, char** tcpOption, int* tcpOptionLen)
{
  int tcpHdrLen = tcpHdr->th_off * sizeof(UINT32);
  int _tcpOptionLen = tcpHdrLen - sizeof(TCP_HDR);
  if (tcpOption != NULL)
    *tcpOption = (char*)tcpHdr + sizeof(TCP_HDR);
  if (tcpOptionLen != NULL)
    *tcpOptionLen = _tcpOptionLen;
  return _tcpOptionLen > 0;
}

//
// All tcpHdr field except tcpHdr.th_sum
// Added ipHdr.ip_src, ipHdr.ip_dst, tcpHdrDataLen and IPPROTO_TCP
//
UINT16 SnoopTCP::checksum(IP_HDR* ipHdr, TCP_HDR* tcpHdr)
{
  int i;
  int tcpHdrDataLen;
  UINT32 src, dst;
  UINT32 sum;
  UINT16 *p;
  
  tcpHdrDataLen = ntohs(ipHdr->ip_len) - sizeof(IP_HDR);
  sum = 0;

  // Add tcpHdr and data buffer as array of UIN16
  p = (UINT16*)(tcpHdr);
  for (i = 0; i < tcpHdrDataLen / 2; i++)
  {
    sum += htons(*p);
    p++;
  }

  // If length is odd, add last data(padding)
  if ((tcpHdrDataLen / 2) * 2 != tcpHdrDataLen)
    sum += (htons(*p) & 0xFF00);

  // Decrease checksum from sum
  sum -= ntohs(tcpHdr->th_sum);

  // Add src address
  src = ntohl(ipHdr->ip_src);
  sum += ((src & 0xFFFF0000) >> 16) + (src & 0x0000FFFF);

  // Add dst address
  dst = ntohl(ipHdr->ip_dst);
  sum += ((dst & 0xFFFF0000) >> 16) + (dst & 0x0000FFFF);

  // Add extra information
  sum += (UINT32)(tcpHdrDataLen) + IPPROTO_TCP;

  // Recalculate sum
  while(sum >> 16)
  {
    sum = (sum & 0xFFFF) + (sum >> 16);
  }
  sum = ~sum;

  return (UINT16)sum;
}

bool SnoopTCP::parse(SnoopPacket* packet)
{
  if (!SnoopIP::parse(packet)) return false;
  
  if (!SnoopIP::isTCP(packet->ipHdr, &packet->tcpHdr)) return false;
  packet->protoType = PROTOTYPE_TCP;

  return true;
}

int SnoopTCP::getOption(
  char*           tcpOption,
  int             tcpOptionLen,
  SnoopTCPOption* snoopTCPOption)
{
  UINT8* p;
  UINT8  kind;
  UINT8  len;
  string desc;

  p = (UINT8*)tcpOption;

  //
  // Set kind
  //
  if (tcpOptionLen < 1) goto _error;
  kind = *p; p++; tcpOptionLen--;
  snoopTCPOption->kind = kind;

  //
  // Set Length and Value
  //
  if (kind != 0 && kind != 1)
  {
    if (tcpOptionLen < 1) goto _error;
    len = *p; p++; tcpOptionLen--;
    snoopTCPOption->len = len;

    if (tcpOptionLen < len - 2) goto _error;
    memcpy(snoopTCPOption->value, p, len - 2);
    p += len - 2; // p++; tcpOptionLen--; // remove warning
  }

  //
  // Set desc
  //
  switch (kind)
  {
     case  0:  desc = "EOL";                                 break;
     case  1:  desc = "NOP";                                 break;
     case  2:  desc = "MSS";                                 break;
     case  3:  desc = "Window Scale";                        break;
     case  4:  desc = "SACK Permitted";                      break;
     case  5:  desc = "SACK";                                break;
     case  6:  desc = "Echo (obsoleted by option 8)";        break;
     case  7:  desc = "Echo Reply (obsoleted by option 8";   break;
     case  8:  desc = "TSOPT - Time Stamp Option";           break;
     case  9:  desc = "Partial Order Connection Permitte";   break;
     case  10: desc = "Partial Order Service Profile";       break;
     case  11: desc = "CC";                                  break;
     case  12: desc = "CC.NEW";                              break;
     case  13: desc = "CC.ECHO";                             break;
     case  14: desc = "TCP Alternate Checksum Request";      break;
     case  15: desc = "TCP Alternate Checksum Data";         break;
     case  16: desc = "Skeeter";                             break;
     case  17: desc = "Bubba";                               break;
     case  18: desc = "Trailer Checksum Option";             break;
     case  19: desc = "MD5 Signature Option";                break;
     case  20: desc = "SCPS Capabilities";                   break;
     case  21: desc = "Selective Negative Acknowledgements"; break;
     case  22: desc = "Record Boundaries";                   break;
     case  23: desc = "Corruption experienced";              break;
     case  24: desc = "SNAP";                                break;
     case  25: desc = "Unassigned (released 12/18/00)";      break;
     case  26: desc = "TCP Compression Filter";              break;
     case  27: desc = "Quick-Start Response";                break;
     case 253: desc = "RFC3692-style Experiment 1";          break;
     case 254: desc = "RFC3692-style Experiment 2";          break;
     default : desc = "UNKNOWN"; return 0;
  }

  snoopTCPOption->desc = desc;

  return (int)(p - (UINT8*)tcpOption);
_error:
  return 0;
}

int SnoopTCP::getOptionList(
  char*               tcpOption,
  int                 tcpOptionLen,
  SnoopTCPOptionList* snoopTCPOptionList)
{
  int totalRes = 0;
  while (true)
  {
    SnoopTCPOption* snoopTCPOption = new SnoopTCPOption;
    int res = getOption(tcpOption, tcpOptionLen, snoopTCPOption);
    if (res == 0) 
    {
      delete snoopTCPOption;
      break;
    }
    snoopTCPOptionList->add(snoopTCPOption);
    tcpOption = (char*)tcpOption + res;
    tcpOptionLen -= res;
    totalRes += res;
  }
  return totalRes;
}

int SnoopTCP::sendRST(pcap* pcap, SnoopPacket* packet)
{
  if (packet->protoType != PROTOTYPE_TCP)
  {
    LOG_ERROR("[SnoopTCP.cpp] SnoopTCP::sendRST packet->protoType is %d", (int)packet->protoType);
    return -1;
  }

	static const int BUF_SIZE = sizeof(ETH_HDR) + sizeof(IP_HDR) + sizeof(TCP_HDR);
	char buf[BUF_SIZE];
	
	ETH_HDR* _ethHdr = (ETH_HDR*) &buf[0];
	IP_HDR*  _ipHdr  = (IP_HDR*)  &buf[sizeof(ETH_HDR)];
	TCP_HDR* _tcpHdr = (TCP_HDR*) &buf[sizeof(ETH_HDR) + sizeof(IP_HDR)];

	//
	// Ethernet
	//
	memcpy(_ethHdr, packet->ethHdr, sizeof(ETH_HDR));

	//
	// IP Header
	//
	memcpy(_ipHdr, packet->ipHdr, sizeof(IP_HDR));
	_ipHdr->ip_tos = 0x44; // value of 44 means tag identifier of Snoop Component RST sending.
	_ipHdr->ip_len = htons(sizeof(IP_HDR) + sizeof(TCP_HDR));

	//
	// TCP Header
	//
	int tcpDataLen = 0;
	if (!isData(packet->ipHdr, packet->tcpHdr, NULL, &tcpDataLen)) tcpDataLen = 0;
	int flagAddValue = ((packet->tcpHdr->th_flags & TH_SYN) || (packet->tcpHdr->th_flags & TH_FIN)) ? 1 : 0;
	UINT32 newSeq = htonl(ntohl(packet->tcpHdr->th_seq) + tcpDataLen + flagAddValue);

	memcpy(_tcpHdr, packet->tcpHdr, sizeof(TCP_HDR));
	_tcpHdr->th_seq   = newSeq;
	_tcpHdr->th_flags = TH_RST | TH_ACK;
	_tcpHdr->th_win   = 0;

	//
	// Checksum
	//
	_tcpHdr->th_sum = htons(checksum(_ipHdr, _tcpHdr));
	_ipHdr->ip_sum = htons(SnoopIP::checksum(_ipHdr));

	int res = pcap_sendpacket(pcap, (const u_char*)buf, BUF_SIZE);
	if (res < 0)
	{
		LOG_ERROR("[SnoopTCP.cpp] SnoopTCP::sendRST pcap_sendpacket return %d", res);
	}
	return res;
}