#include "SnoopType.h"

// ----------------------------------------------------------------------------
// MAC
// ----------------------------------------------------------------------------
MAC& MAC::operator = (const string rhs)
{
  int i;
  UCHAR *p;
  UCHAR ch1, ch2;

  p = (UCHAR*)rhs.c_str();
  for (i = 0 ; i < SNOOP_MAC_SIZE; i++)
  {
    ch1 = *p++;
    if (ch1 >= 'a')
      ch1 = ch1 - 'a' + 10;
    else
    if (ch1 >= 'A')
      ch1 = ch1 - 'A' + 10;
    else
      ch1 = ch1 - '0';
    ch2 = *p++;
    if (ch2 >= 'a')
      ch2 = ch2 - 'a' + 10;
    else if (ch2 >= 'A')
      ch2 = ch2 - 'A' + 10;
    else
      ch2 = ch2 - '0';
    s[i] = (ch1 << 4) + ch2;
    while (*p == '-' || *p == ':') p++;
  }
  return *this;
}

MAC& MAC::operator = (const UINT8* rhs)
{
  memcpy(s, rhs, SNOOP_MAC_SIZE);
  return *this;
}

MAC::operator string()
{
  UCHAR ch1, ch2;
  int i, index;
  char buf[SNOOP_MAC_SIZE * 3]; // enough size

  index = 0;
  for (i = 0; i < SNOOP_MAC_SIZE; i++)
  {
    ch1 = s[i] & 0xF0;
    ch1 = ch1 >> 4;
    if (ch1 > 9)
      ch1 = ch1 + 'A' - 10;
    else
      ch1 = ch1 + '0';
    ch2 = s[i] & 0x0F;
    if (ch2 > 9)
      ch2 = ch2 + 'A' - 10;
    else
      ch2 = ch2 + '0';
    buf[index++] = ch1;
    buf[index++] = ch2;
    if (i == 2)
      buf[index++] = '-';
  }
  buf[index] = '\0';
  return (string(buf));
}

string MAC::str()
{
  return (string)(*this);
}

string MAC::format(const string format)
{
  return ::format(format.c_str(), s[0], s[1], s[2], s[3], s[4], s[5]);
}

bool MAC::operator == (const MAC& rhs) const
{
  return memcmp(s, rhs.s, SNOOP_MAC_SIZE) == 0;
}

bool MAC::operator != (const MAC& rhs) const
{
  return memcmp(s, rhs.s, SNOOP_MAC_SIZE) != 0;
}

bool MAC::operator < (const MAC& rhs) const
{
  return memcmp(s, rhs.s, SNOOP_MAC_SIZE) < 0;
}

bool MAC::operator > (const MAC& rhs) const
{
  return memcmp(s, rhs.s, SNOOP_MAC_SIZE) > 0;
}

bool MAC::operator <= (const MAC& rhs) const
{
  return memcmp(s, rhs.s, SNOOP_MAC_SIZE) <= 0;
}

bool MAC::operator >= (const MAC& rhs) const
{
  return memcmp(s, rhs.s, SNOOP_MAC_SIZE) >= 0;
}

bool MAC::operator == (const UINT8 rhs[]) const
{
  return memcmp(s, rhs, SNOOP_MAC_SIZE) == 0;
}

void MAC::clear()
{
  *this = snoopCleanMac;
}

bool MAC::isClear()
{
  return *this == snoopCleanMac;
}

MAC MAC::getRandomMac()
{
  MAC res;
  for (int i = 0; i < SNOOP_MAC_SIZE; i++)
  {
    res.s[i] = rand() % 256;
  }
  res.s[0] &= 0x7F;
  return res;
}

UINT8 __snoopCleanMac[] =  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
MAC snoopCleanMac(__snoopCleanMac);

UINT8 __snoopBroadcastMac[] =  { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
MAC snoopBroadcastMac(__snoopBroadcastMac);
