// ----------------------------------------------------------------------------
//
// Snoop Component Suite version 5.0
//
// http://www.gilgil.net
//
// Copyright (c) NetworkLab Corporation All rights reserved
//
// ----------------------------------------------------------------------------

#ifndef SnoopTypeH
#define SnoopTypeH

#include "SnoopCommon.h"

#pragma pack(push, 1)
#include "Libnet/config.h"
#include "Libnet/libnet-macros.h"
#include "Libnet/libnet-headers.h"
#pragma pack(pop)

// ----------------------------------------------------------------------------
// PKT_HDR
// ----------------------------------------------------------------------------
typedef struct pcap_pkthdr             PKT_HDR;

// ----------------------------------------------------------------------------
// MAC
// ----------------------------------------------------------------------------
#define SNOOP_MAC_SIZE 6

#pragma pack(push, 1)
class MAC
{
public:
  UINT8 s[SNOOP_MAC_SIZE];

  MAC()                   {}
  MAC(const string rhs)   { *this = rhs; }
  MAC(const UINT8 rhs[])  { *this = rhs; }

  //
  // Assign operator
  //
  MAC& operator = (const string rhs);
  MAC& operator = (const UINT8 rhs[]);

  //
  // Type casting operator
  //
  operator string();
  string   str();
  string   format(const string format);

  //
  // Compare operator
  //
  bool operator == (const MAC& rhs) const;
  bool operator != (const MAC& rhs) const;
  bool operator <  (const MAC& rhs) const;
  bool operator >  (const MAC& rhs) const;
  bool operator <= (const MAC& rhs) const;
  bool operator >= (const MAC& rhs) const;
  bool operator == (const UINT8 rhs[]) const;

  void clear();
  bool isClear();
  static MAC  getRandomMac();
};
#pragma pack(pop)

extern MAC snoopCleanMac;
extern MAC snoopBroadcastMac;

// ----------------------------------------------------------------------------
// IP
// ----------------------------------------------------------------------------
typedef UINT32 IP;

// ----------------------------------------------------------------------------
// Header
// ----------------------------------------------------------------------------
#pragma pack(push, 1)

// ----------------------------------------------------------------------------
// ETH_HDR
// ----------------------------------------------------------------------------
typedef struct // libnet_ethernet_hdr
{
    MAC ether_dhost;      /* destination ethernet address */
    MAC ether_shost;      /* source ethernet address */
    u_int16_t ether_type; /* protocol */
} ETH_HDR;

// ----------------------------------------------------------------------------
// FDDI_HDR
// ----------------------------------------------------------------------------
typedef struct libnet_fddi_hdr FDDI_HDR;

// ----------------------------------------------------------------------------
// IP_HDR
// ----------------------------------------------------------------------------
typedef struct IP_HDR // libnet_ipv4_hdr
{
#if (LIBNET_LIL_ENDIAN)
    u_int8_t ip_hl:4,      /* header length */
           ip_v:4;         /* version */
#endif
#if (LIBNET_BIG_ENDIAN)
    u_int8_t ip_v:4,       /* version */
           ip_hl:4;        /* header length */
#endif
    u_int8_t ip_tos;       /* type of service */
#ifndef IPTOS_LOWDELAY
#define IPTOS_LOWDELAY      0x10
#endif
#ifndef IPTOS_THROUGHPUT
#define IPTOS_THROUGHPUT    0x08
#endif
#ifndef IPTOS_RELIABILITY
#define IPTOS_RELIABILITY   0x04
#endif
#ifndef IPTOS_LOWCOST
#define IPTOS_LOWCOST       0x02
#endif
    u_int16_t ip_len;         /* total length */
    u_int16_t ip_id;          /* identification */
    u_int16_t ip_off;
#ifndef IP_RF
#define IP_RF 0x8000          /* reserved fragment flag */
#endif
#ifndef IP_DF
#define IP_DF 0x4000          /* dont fragment flag */
#endif
#ifndef IP_MF
#define IP_MF 0x2000          /* more fragments flag */
#endif 
#ifndef IP_OFFMASK
#define IP_OFFMASK 0x1fff     /* mask for fragmenting bits */
#endif
    u_int8_t ip_ttl;          /* time to live */
    u_int8_t ip_p;            /* protocol */
    u_int16_t ip_sum;         /* checksum */
    u_int32_t ip_src, ip_dst;  /* source and dest address */
} IP_HDR;

// ----------------------------------------------------------------------------
// ARP_HDR
// ----------------------------------------------------------------------------
typedef struct libnet_arp_hdr _ARP_HDR;
typedef struct ARP_HDR : _ARP_HDR
{
  MAC ar_sa;
  IP  ar_si;
  MAC ar_ta;
  IP  ar_ti;
} ARP_HDR;

// ----------------------------------------------------------------------------
// TCP_HDR
// ----------------------------------------------------------------------------
typedef struct libnet_tcp_hdr TCP_HDR;

// ----------------------------------------------------------------------------
// UDP_HDR
// ----------------------------------------------------------------------------
typedef struct libnet_udp_hdr UDP_HDR;

// ----------------------------------------------------------------------------
// ICMP_HDR
// ----------------------------------------------------------------------------
typedef struct libnet_icmpv4_hdr ICMP_HDR;

// ----------------------------------------------------------------------------
// DNS_HDR
// ----------------------------------------------------------------------------
typedef struct DNS_HDR
{
    u_int16_t id;             /* DNS packet ID */
    u_int16_t flags;          /* DNS flags */
    u_int16_t num_q;          /* Number of questions */
    u_int16_t num_answ_rr;    /* Number of answer resource records */
    u_int16_t num_auth_rr;    /* Number of authority resource records */
    u_int16_t num_addi_rr;    /* Number of additional resource records */
} DNS_HDR;

#pragma pack (pop)

#endif  SnoopTypeH